#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from woost.models.file import File
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import content_renderers
from woost.models.rendering.formats import formats_by_mime_type


class ImageFileRenderer(ContentRenderer):
    """A content renderer that handles image files."""

    def can_render(self, item):
        return isinstance(item, File) \
        and item.resource_type == "image" \
        and item.mime_type in formats_by_mime_type

    def render(self, item):
        return item.file_path

    def last_change_in_appearence(self, item):
        return max(
            os.stat(item.file_path).st_mtime,
            ContentRenderer.last_change_in_appearence(self, item)
        )

content_renderers.register(ImageFileRenderer())




#-*- coding: utf-8 -*-
"""
Provides a variety of models to implement customized offers, discounts, and
other specialized pricing policies for a shop.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from datetime import datetime
from cocktail import schema
from cocktail.controllers.usercollection import UserCollection
from woost.models import Item, Site
from woost.extensions.shop.shoporder import ShopOrder
from woost.extensions.shop.shoporderentry import ShopOrderEntry


class PricingPolicy(Item):

    visible_from_root = False
    integral = True
    instantiable = False

    edit_controller = \
        "woost.extensions.shop.pricingpolicyfieldscontroller." \
        "PricingPolicyFieldsController"

    edit_view = "woost.extensions.shop.PricingPolicyFields"

    members_order = [
        "title",
        "enabled",
        "start_date",
        "end_date"
    ]

    title = schema.String(
        translated = True,
        descriptive = True
    )

    enabled = schema.Boolean(
        required = True,
        default = True
    )

    start_date = schema.DateTime(
        indexed = True
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date
    )

    matching_items = schema.Mapping()

    # TODO: Validate issubclass(matching_items["type"], (ShopOrder, Product))

    def is_current(self):
        return (self.start_date is None or self.start_date <= datetime.now()) \
           and (self.end_date is None or self.end_date > datetime.now())

    def select_matching_items(self, *args, **kwargs):
        user_collection = UserCollection(Item)
        user_collection.allow_paging = False
        user_collection.allow_member_selection = False
        user_collection.allow_language_selection = False
        user_collection.params.source = self.matching_items.get
        #user_collection.available_languages = Language.codes # <- required?
        return user_collection.subset

    def match_item(self, item):

        if self.matching_items:
            for filter in self.select_matching_items().filters:
                if not filter.eval(item):
                    return False

        return True

    def applies_to(self, item):
        return self.enabled and self.is_current() and self.match_item(item)

    def apply(self, item, costs):
        pass


class Discount(PricingPolicy):
    pass


class PriceOverride(Discount):

    instantiable = True

    price = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["price"]["cost"] = self.price


class RelativeDiscount(Discount):

    instantiable = True

    amount = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["price"]["cost"] -= self.amount


class PercentageDiscount(Discount):

    instantiable = True

    percentage = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["price"]["percentage"] -= self.percentage


class FreeUnitsDiscount(Discount):

    instantiable = True
    members_order = ["paid_units", "free_units", "repeated"]

    members_order = [
        "paid_units",
        "free_units",
        "repeated"
    ]

    paid_units = schema.Integer(
        required = True,
        min = 0
    )

    free_units = schema.Integer(
        required = True,
        min = 1
    )

    repeated = schema.Boolean(
        required = True,
        default = True
    )

    def apply(self, item, costs):

        paid = self.paid_units
        free = self.free_units
        quantity = costs["paid_quantity"]

        if self.repeated:
            quantity -= (quantity / (paid + free)) * free
        elif quantity > paid:
            quantity = max(paid, quantity - free)

        costs["paid_quantity"] = quantity


class ShippingCost(PricingPolicy):
    pass


class ShippingCostOverride(ShippingCost):

    instantiable = True

    cost = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["shipping"] = self.cost


class CumulativeShippingCost(ShippingCost):

    instantiable = True

    cost = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["shipping"] += self.cost


class Tax(PricingPolicy):
    pass


class CumulativeTax(Tax):

    instantiable = True

    cost = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["tax"]["cost"] += self.cost


class PercentageTax(Tax):

    instantiable = True

    percentage = schema.Decimal(
        required = True
    )

    def apply(self, item, costs):
        costs["tax"]["percentage"] += self.percentage



#-*- coding: utf-8 -*-
u"""Defines the `BlocksPage` model.

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import Document, Template
from woost.extensions.blocks.block import Block


class BlocksPage(Document):

    blocks = schema.Collection(
        items = schema.Reference(type = Block),
        related_end = schema.Collection(),
        member_group = "content"
    )

    default_template = schema.DynamicDefault(
        lambda: Template.get_instance(
            qname = "woost.extensions.blocks.blocks_page_template"
        )
    )




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from cocktail import schema
from woost.models import (
    Controller,
    Document,
    EmailTemplate,
    Publishable,
    Role,
    Template,
    User
)

def excluded_roles():
    return Role.qname.not_one_of((
        'woost.anonymous',
        'woost.everybody',
        'woost.authenticated'
    ))

class SignUpPage(Document):

    members_order = [
        "user_type",
        "roles",
        "confirmation_target",
        "confirmation_email_template"
    ]

    # Defines the persistent class that will be
    # used like schema in signup process
    user_type = schema.Reference(
        class_family = User,
        required = True,
        member_group = "signup_process"
    )

    # The collection of roles that will be applyed
    # to each instance (of user_type class) created throw
    # a signuppage
    roles = schema.Collection(
        items = "woost.models.Role",
        related_end = schema.Collection(
            name = "related_signup_pages",
            visible = False
        ),
        edit_inline = True,
        member_group = "signup_process",
        relation_constraints = lambda ctx: [excluded_roles()]
    )

    # If is None, doesn't require an email confirmation
    # process to complete signup process
    confirmation_email_template = schema.Reference(
        type = EmailTemplate,
        related_end = schema.Collection(),
        member_group = "signup_process"
    )

    confirmation_target = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection(),
        member_group = "signup_process",
        required = True
    )

    default_template = schema.DynamicDefault(
        lambda: Template.get_instance(qname = u"woost.extensions.signup.signup_template")
    )

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = u"woost.extensions.signup.signup_controller")
    )

    default_confirmation_email_template = schema.DynamicDefault(
        lambda: EmailTemplate.get_instance(qname = u"woost.extensions.signup.signup_confirmation_email_template")
    )

    default_confirmation_target = schema.DynamicDefault(
        lambda: Publishable.get_instance(qname = u"woost.extensions.signup.signup_confirmation_target")
    )



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.modeling import cached_getter
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.shop.basket import Basket


class ShopController(DocumentController):

    @cached_getter
    def output(self):
        output = DocumentController.output(self)
        output["shop_order"] = Basket.get()
        return output




#-*- coding: utf-8 -*-
u"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         March 2010
"""
from cocktail import schema
from cocktail.events import event_handler
from cocktail.persistence import datastore
from woost.models import Item, StandardPage

class CampaignMonitorList(Item):

    visible_from_root = False

    members_order = [
        "list_id",
        "pending_page",
        "confirmation_success_page",
        "unsubscribe_page",
    ]

    default_pending_page = schema.DynamicDefault(
        lambda: StandardPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.pending_page"
        )
    )

    title = schema.String(
        editable = False,
        listed_by_default = False,
        descriptive = True
    )

    list_id = schema.String(
        unique = True,
        editable = False,
        text_search = False
    )

    unsubscribe_page = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )

    pending_page = schema.Reference(
        required = True,
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )

    confirmation_success_page = schema.Reference(
        type = "woost.models.Publishable",
        related_end = schema.Collection()
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail.styled import styled

def permission_separator_style(char = u"-", width = 80):
    return styled(char * width, "violet")

def permission_check_style(t, width = 80):
    return u"\n" + styled(t.ljust(width, " "), "white", "violet")

def permission_param_style(key, value):
    return u" " * 4 + styled(key + ":", style = "bold") \
         + u" " + unicode(value)

def role_style(t):
    return u"\n" + (u" " * 4) + styled(t, style = "underline")

def permission_style(t):
    return u" " * 4 + styled(t, "light_gray")

def authorized_style(t):
    return u" " * 4 + styled(t.upper(), "white", "green") + u"\n"

def unauthorized_style(t):
    return u" " * 4 + styled(t.upper(), "white", "red") + u"\n"

def permission_doesnt_match_style(t):
    return u" " * 4 + styled(t, "yellow") + u"\n"

def trigger_style(t):
    return styled(t.ljust(80), "white", "brown")

def trigger_context_style(k, v):
    return styled(" " * 4 + (k + ":").ljust(15), style = "bold") \
        + " " + unicode(v)

def trigger_doesnt_match_style(t):
    return styled(" " * 4 + t, "yellow")

def trigger_match_style(t):
    return " " * 4 + styled(t.upper(), "white", "green")




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import cherrypy
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.modeling import cached_getter
from cocktail.controllers.location import Location
from cocktail.controllers.formcontrollermixin import FormControllerMixin
from woost.controllers.documentcontroller import DocumentController
from woost.controllers.backoffice.usereditnode import PasswordConfirmationError
from woost.extensions.signup.signupconfirmationcontroller import generate_confirmation_hash
from woost.extensions.signup.signuppage import SignUpPage

from woost.models import Item, Site, Language, User

class SignUpController(FormControllerMixin, DocumentController):

    def __init__(self, *args, **kwargs):
        DocumentController.__init__(self, *args, **kwargs)
        FormControllerMixin.__init__(self)

    @cached_getter
    def form_model(self):
        return self.context["publishable"].user_type

    @cached_getter
    def form_adapter(self):

        adapter = FormControllerMixin.form_adapter(self)
        adapter.exclude(
            ["prefered_language",
             "roles",
             "password_confirmation",
             "enabled",
             "confirmed_email"]
            + [key for key in Item.members()]
        )
        return adapter

    @cached_getter
    def form_schema(self):

        form_schema = FormControllerMixin.form_schema(self)

        # Set schema name in order to keep always the same value
        # although change value of form_model member
        form_schema.name = u"SignUpForm"

        # Adding extra field for password confirmation
        if form_schema.get_member("password"):

            password_confirmation_member = schema.String(
                name = "password_confirmation",
                edit_control = "cocktail.html.PasswordBox",
                required = form_schema.get_member("password")
            )
            form_schema.add_member(password_confirmation_member)

            # Set password_confirmation position just after
            # password member position
            order_list = form_schema.members_order
            pos = order_list.index("password")
            order_list.insert(pos + 1, "password_confirmation")

            # Add validation to compare password_confirmation and
            # password fields
            @password_confirmation_member.add_validation
            def validate_password_confirmation(member, value, ctx):
                password = ctx.get_value("password")
                password_confirmation = value

                if password and password_confirmation \
                and password != password_confirmation:
                    yield PasswordConfirmationError(
                            member, value, ctx)

        return form_schema

    def submit(self):

        FormControllerMixin.submit(self)
        publishable = self.context["publishable"]

        # Adding roles
        instance = self.form_instance
        instance.roles.extend(publishable.roles)

        # If require email confirmation, disabled authenticated access
        # and send email confirmation message
        confirmation_email_template = publishable.confirmation_email_template
        if confirmation_email_template:
            instance.enabled = False
            instance.confirmed_email = False
            confirmation_email_template.send(self.email_parameters)
            # Storing instance
            instance.insert()
            datastore.commit()
        else:
            # Storing instance
            instance.insert()
            datastore.commit()
            # Getting confirmation target uri
            confirmation_target = self.context["publishable"].confirmation_target
            uri = self.context["cms"].uri(confirmation_target)
            # Enabling user and autologin
            instance.enabled = True
            self.context["cms"].authentication.set_user_session(instance)
            # Redirecting to confirmation target
            raise cherrypy.HTTPRedirect(uri)

    @cached_getter
    def email_parameters(self):
        parameters = {
            "user": self.form_instance,
            "confirmation_url": self.confirmation_url
        }
        return parameters

    @cached_getter
    def confirmation_url(self):
        instance = self.form_instance
        confirmation_target = self.context["publishable"].confirmation_target
        canonical_uri = self.context["cms"].uri(confirmation_target)
        location = Location.get_current(relative=False)
        location.path_info = canonical_uri
        location.query_string = {
            "email": instance.email,
            "hash": generate_confirmation_hash(instance.email)
        }
        return str(location)



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block


class TwitterTimelineBlock(Block):

    instantiable = True
    view_class = "cocktail.html.TwitterTimeline"

    members_order = [
        "account",
        "max_tweets"
    ]

    account = schema.String(
        required = True,
        member_group = "content"
    )

    max_tweets = schema.Integer(
        required = True,
        default = 5,
        min = 1,
        max = 20,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.account = self.account
        view.max_tweets = self.max_tweets




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import sys
import cherrypy
from cocktail.iteration import first
from cocktail.events import event_handler
from cocktail.translations import translations, language_context
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import (
    request_property,
    get_parameter,
    Location
)
from woost.models import Publishable, get_current_user
from woost.controllers.notifications import notify_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.opengraph import OpenGraphExtension
from woost.extensions.facebookpublication import FacebookPublicationExtension
from woost.extensions.facebookpublication.facebookpublicationtarget \
    import FacebookPublicationTarget
from woost.extensions.facebookpublication.facebookpublicationpermission \
    import FacebookPublicationPermission


class FacebookPublicationController(BaseBackOfficeController):

    notice_summary_threshold = 4
    view_class = "woost.extensions.facebookpublication.FacebookPublicationView"

    @request_property
    def form_schema(self):
        return schema.Schema("FacebookPublicationForm", members = [
            schema.Collection("subset",
                items = schema.Reference(
                    type = Publishable,
                    required = True,
                    enumeration = lambda ctx: self.selection
                ),
                min = 1,
                default = schema.DynamicDefault(lambda: self.selection)
            ),
            schema.Collection("published_languages",
                items = schema.String(
                    translate_value = lambda value, language = None, **kwargs:
                        "" if not value
                           else translations(value, language, **kwargs),
                    enumeration = lambda ctx: self.eligible_languages
                ),
                min = 1,
                default = schema.DynamicDefault(
                    lambda: self.eligible_languages
                )
            ),
            schema.Collection("publication_targets",
                items = schema.Reference(
                    type = FacebookPublicationTarget,
                    required = True,
                    enumeration = lambda ctx: self.allowed_publication_targets
                ),
                min = 1,
                default = schema.DynamicDefault(
                    lambda: self.allowed_publication_targets
                )
            )
        ])

    @request_property
    def form_data(self):
        return get_parameter(
            self.form_schema,
            errors = "ignore",
            undefined = "set_none" if self.submitted else "set_default"
        )

    @request_property
    def form_errors(self):
        return schema.ErrorList(
            []
            if not self.submitted
            else self.form_schema.get_errors(self.form_data)
        )

    @event_handler
    def handle_before_request(cls, e):

        controller = e.source

        if not controller.allowed_publication_targets:
            raise cherrypy.HTTPError(403, "Forbidden")

    @request_property
    def allowed_publication_targets(self):

        from woost.extensions.facebookpublication \
            import FacebookPublicationExtension

        user = get_current_user()

        return [fb_target
                for fb_target in FacebookPublicationExtension.instance.targets
                if fb_target.auth_token
                and all(
                    user.has_permission(
                        FacebookPublicationPermission,
                        target = publishable,
                        publication_target = fb_target
                    )
                    for publishable in self.selection
                )]

    @request_property
    def eligible_languages(self):

        selection_languages = set()

        for publishable in self.selection:
            selection_languages.update(publishable.translations.iterkeys())

        target_languages = set()

        for target in self.allowed_publication_targets:
            target_languages.update(target.languages)

        return selection_languages & target_languages

    @request_property
    def selection(self):
        return get_parameter(
            schema.Collection("selection",
                items = schema.Reference(
                    type = Publishable,
                    required = True
                ),
                min = 1
            ),
            errors = "raise"
        )

    @request_property
    def submitted(self):
        return cherrypy.request.method == "POST"

    @request_property
    def valid(self):
        return self.action != "publish" \
            or self.form_schema.validate(self.form_data)

    @request_property
    def action(self):
        return cherrypy.request.params.get("action")

    def submit(self):

        if self.action == "close":
            self.go_back()

        form_data = self.form_data
        subset = form_data["subset"]
        languages = set(form_data["published_languages"])
        targets = form_data["publication_targets"]
        check = (self.action == "check")

        results = self.results

        if check:
            og = OpenGraphExtension.instance

        for target in targets:

            if check:
                posts = target.feed_posts()

            for publishable in subset:
                for language in sorted(languages & set(target.languages)):
                    if check:
                        try:
                            with language_context(language):
                                uri = og.get_properties(publishable)["og:url"]

                            results.append((
                                publishable,
                                target,
                                language,
                                first(
                                    post
                                    for post in posts
                                    if post.get("link") == uri
                                )
                            ))
                        except Exception, ex:
                            results.append((publishable, target, language, ex))
                    else:
                        try:
                            with language_context(language):
                                target.publish(publishable)
                        except Exception, ex:
                            results.append((publishable, target, language, ex))
                        else:
                            results.append((publishable, target, language, None))

    @request_property
    def results(self):
        return []

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            form_schema = self.form_schema,
            form_data = self.form_data,
            form_errors = self.form_errors,
            results = self.results,
            action = self.action
        )
        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.translations import translations

translations.define("woost.extensions.sitemap.sitemap_controller.title",
    ca = u"Controlador de mapa web",
    es = u"Controlador de mapa web",
    en = u"Sitemap controller"
)

translations.define("woost.extensions.sitemap.sitemap_document.title",
    ca = u"Mapa XML del lloc web",
    es = u"Mapa XML del sitio web",
    en = u"XML sitemap"
)

translations.define("Publishable.sitemap",
    ca = u"Refinament de l'indexat web",
    es = u"Refinamiento del indexado web",
    en = u"Web crawler hints"
)

translations.define("Publishable.sitemap_indexable",
    ca = u"Indexable",
    es = u"Indexable",
    en = u"Indexable"
)

translations.define("Publishable.sitemap_change_frequency",
    ca = u"Freqüència d'actualització",
    es = u"Frecuencia de actualización",
    en = u"Change frequency"
)

translations.define("Publishable.sitemap_priority",
    ca = u"Prioritat",
    es = u"Prioridad",
    en = u"Priority"
)

translations.define("Publishable.sitemap_priority-explanation",
    ca = u"Prioritat relativa de l'element respecte la resta de pàgines del "
         u"lloc web. S'expressa com un valor decimal entre 0 i 1.",
    es = u"Prioridad relativa del elemento respecto al resto de páginas del "
         u"sitio web. Se expresa como un valor decimal entre 0 y 1.",
    en = u"Relative priority of the element, relative to other site content. "
         u"Expressed as a decimal number, ranging from 0 to 1."
)

translations.define("woost.extensions.sitemap.change_frequency always",
    ca = u"Sempre",
    es = u"Siempre",
    en = u"Always"
)

translations.define("woost.extensions.sitemap.change_frequency hourly",
    ca = u"Cada hora",
    es = u"Cada hora",
    en = u"Hourly"
)

translations.define("woost.extensions.sitemap.change_frequency daily",
    ca = u"Diària",
    es = u"Diaria",
    en = u"Daily"
)

translations.define("woost.extensions.sitemap.change_frequency weekly",
    ca = u"Setmanal",
    es = u"Semanal",
    en = u"Weekly"
)

translations.define("woost.extensions.sitemap.change_frequency monthly",
    ca = u"Mensual",
    es = u"Mensual",
    en = u"Monthly"
)

translations.define("woost.extensions.sitemap.change_frequency yearly",
    ca = u"Anual",
    es = u"Anual",
    en = u"Yearly"
)

translations.define("woost.extensions.sitemap.change_frequency never",
    ca = u"Mai",
    es = u"Nunca",
    en = u"Never"
)




#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
A script that imports a set of local files as CMS items.
"""
from sys import argv, exit
from glob import glob
from pkg_resources import resource_filename
from cocktail.styled import styled
from cocktail.persistence import datastore
from woost.models import File
import _PROJECT_NAME_

def main():

    params = argv[1:]

    if not params:
        print "Usage: %s filelist" % argv[0]
        exit(1)

    for param in params:
        for path in glob(param):
            print "Importing " + styled(path, style = "bold") + "...",
            try:
                file = File.from_path(
                    path,
                    resource_filename("_PROJECT_NAME_", "upload")
                )
                file.insert()
            except Exception, ex:
                print styled(str(ex), "red")
            else:
                print styled("File #" + str(file.id), "green")

    datastore.commit()

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.modeling import cached_getter
from woost.controllers.backoffice.editstack import EditNode


class PublishableEditNode(EditNode):

    @cached_getter
    def form_adapter(self):
        adapter = EditNode.form_adapter(self)
        adapter.exclude([
            "enabled"
            if self.item.per_language_publication
            else "translation_enabled"
        ])
        return adapter




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from pkg_resources import iter_entry_points
from threading import RLock
from ZODB.POSException import ConflictError
from cocktail.iteration import first
from cocktail.modeling import classgetter
from cocktail.events import Event, event_handler
from cocktail.pkgutils import resolve
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore, transaction
from woost.models.item import Item
from woost.models.language import Language

extension_translations = object()
_loaded_extensions = set()
_extensions_lock = RLock()

def load_extensions():
    """Load all available extensions.

    This is tipically called during application start up, and follows this
    sequence:

        * New available extensions are installed
        * Previously installed exceptions that are no longer available are
          uninstalled
        * All installed and enabled extensions are initialized
    """
    with _extensions_lock:

        for entry_point in iter_entry_points("woost.extensions"):

            extension_type = entry_point.load()
            extension = extension_type.instance

            # Create an instance of each new extension
            if extension is None:
                def create_extension_instance():
                    extension = extension_type()
                    extension.insert()

                transaction(
                    create_extension_instance,
                    max_attempts = 5,
                    desist = lambda: extension_type.instance is not None
                )

            # Load enabled extensions
            elif extension.enabled:
                extension.load()


class Extension(Item):
    """Base model for Woost extensions."""

    instantiable = False
    collapsed_backoffice_menu = True
    edit_node_class = "woost.controllers.backoffice.extensioneditnode." \
                      "ExtensionEditNode"

    installed = False

    @property
    def loaded(self):
        return self.__class__ in _loaded_extensions

    member_order = (
        "extension_author",
        "license",
        "web_page",
        "description",
        "enabled"
    )

    extension_author = schema.String(
        editable = False,
        listed_by_default = False
    )

    license = schema.String(
        editable = False,
        listed_by_default = False
    )

    web_page = schema.String(
        editable = False
    )

    description = schema.String(
        editable = False,
        translated = True
    )

    enabled = schema.Boolean(
        required = True,
        default = False
    )

    loading = Event("""An event triggered during application start up.""")

    installing = Event(
        """An event triggered when an extension creates its assets, the first
        time it is loaded.
        """
    )

    @classgetter
    def instance(cls):
        return first(cls.select())

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.name)

    def load(self):
        with _extensions_lock:
            if self.__class__ not in _loaded_extensions:
                _loaded_extensions.add(self.__class__)
                self._load()
                self.loading()

    def _load(self):
        pass

    def install(self):
        if not self.installed:
            def install_extension():
                self._install()
                self.installing()
                self.installed = True

            transaction(install_extension, desist = lambda: self.installed)

    def _install(self):
        pass

    def _create_asset(self, cls, id, **values):
        """Convenience method for creating content when installing an
        extension.
        """
        asset = cls()
        cls.qname = qname = self.full_name.rsplit(".", 1)[0] + "." + id

        if values:
            for key, value in values.iteritems():
                if value is extension_translations:
                    for language in Language.codes:
                        value = translations(qname + "." + key, language)
                        if value:
                            asset.set(key, value, language)
                else:
                    asset.set(key, value)

        asset.insert()
        return asset




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.translations import translations

# ShopExtension
#------------------------------------------------------------------------------
translations.define("ShopExtension.discounts",
    ca = u"Descomptes",
    es = u"Descuentos",
    en = u"Discounts"
)

translations.define("ShopExtension.shipping_costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs"
)

translations.define("ShopExtension.taxes",
    ca = u"Impostos",
    es = u"Impuestos",
    en = u"Taxes"
)

# Product
#------------------------------------------------------------------------------
translations.define("Product",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product"
)

translations.define("Product-plural",
    ca = u"Productes",
    es = u"Productos",
    en = u"Products"
)

translations.define("Product.price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price"
)

translations.define("Product.categories",
    ca = u"Categories",
    es = u"Categorías",
    en = u"Categories"
)

# ProductCategory
#------------------------------------------------------------------------------
translations.define("ProductCategory",
    ca = u"Categoria de producte",
    es = u"Categoría de producto",
    en = u"Product category"
)

translations.define("ProductCategory-plural",
    ca = u"Categories de producte",
    es = u"Categorías de producto",
    en = u"Product categories"
)

translations.define("ProductCategory.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("ProductCategory.parent",
    ca = u"Categoria pare",
    es = u"Categoría padre",
    en = u"Parent category"
)

translations.define("ProductCategory.children",
    ca = u"Categories filles",
    es = u"Categorías hijas",
    en = u"Child categories"
)

translations.define("ProductCategory.products",
    ca = u"Productes",
    es = u"Productos",
    en = u"Products"
)

# ShopOrder
#------------------------------------------------------------------------------
translations.define("ShopOrder",
    ca = u"Comanda",
    es = u"Pedido",
    en = u"Shop order"
)

translations.define("ShopOrder-plural",
    ca = u"Comandes",
    es = u"Pedidos",
    en = u"Shop orders"
)

translations.define("ShopOrder.entries",
    ca = u"Contingut de la comanda",
    es = u"Contenido del pedido",
    en = u"Order entries"
)

translations.define("ShopOrder.cost",
    ca = u"Cost",
    es = u"Coste",
    en = u"Cost"
)

translations.define("ShopOrder.status",
    ca = u"Estat",
    es = u"Estado",
    en = u"State"
)

translations.define("woost.extensions.shop.ShopOrder.status pending",
    ca = u"Pendent",
    es = u"Pendiente",
    en = u"Pending"
)

translations.define("woost.extensions.shop.ShopOrder.status accepted",
    ca = u"Acceptada",
    es = u"Aceptada",
    en = u"Accepted"
)

translations.define("woost.extensions.shop.ShopOrder.status failed",
    ca = u"Cancel·lada",
    es = u"Cancelada",
    en = u"Cancelled"
)

translations.define("ShopOrder.language",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language"
)

translations.define("ShopOrder.shipping_info",
    ca = u"Informació d'enviament",
    es = u"Información de envío",
    en = u"Shipping address"
)

translations.define("ShopOrder.address",
    ca = u"Adreça d'enviament",
    es = u"Dirección de envío",
    en = u"Shipping address"
)

translations.define("ShopOrder.town",
    ca = u"Població",
    es = u"Población",
    en = u"Town"
)

translations.define("ShopOrder.region",
    ca = u"Regió/Estat/Província",
    es = u"Región/Estado/Provincia",
    en = u"Region/Estado/Provincia"
)

translations.define("ShopOrder.country",
    ca = u"País",
    es = u"País",
    en = u"Country"
)

translations.define("ShopOrder.postal_code",
    ca = u"Codi postal",
    es = u"Código postal",
    en = u"Postal code"
)

# ShopOrderEntry
#------------------------------------------------------------------------------
translations.define("ShopOrderEntry",
    ca = u"Línia de comanda",
    es = u"Linea de pedido",
    en = u"Shop order entry"
)

translations.define("ShopOrderEntry-plural",
    ca = u"Línies de comanda",
    es = u"Lineas de pedido",
    en = u"Shop order entries"
)

translations.define("ShopOrderEntry.shop_order",
    ca = u"Comanda",
    es = u"Pedido",
    en = u"Shop order"
)

translations.define("ShopOrderEntry.product",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product"
)

translations.define("ShopOrderEntry.quantity",
    ca = u"Quantitat",
    es = u"Cantidad",
    en = u"Quantity"
)

translations.define("ShopOrderEntry.cost",
    ca = u"Cost",
    es = u"Coste",
    en = u"Cost"
)

# PricingPolicy
#------------------------------------------------------------------------------
translations.define("PricingPolicy",
    ca = u"Política de preus",
    es = u"Política de precios",
    en = u"Pricing policy"
)

translations.define("PricingPolicy-plural",
    ca = u"Polítiques de preus",
    es = u"Políticas de precios",
    en = u"Pricing policies"
)

translations.define("PricingPolicy.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("PricingPolicy.enabled",
    ca = u"Activa",
    es = u"Activa",
    en = u"Enabled"
)

translations.define("PricingPolicy.start_date",
    ca = u"Data d'inici",
    es = u"Fecha de inicio",
    en = u"Start date"
)

translations.define("PricingPolicy.end_date",
    ca = u"Data de fi",
    es = u"Fecha de fin",
    en = u"End date"
)

translations.define("PricingPolicy.matching_items",
    ca = u"Elements afectats",
    es = u"Elementos afectados",
    en = u"Matching items"
)

# Discount
#------------------------------------------------------------------------------
translations.define("Discount",
    ca = u"Descompte",
    es = u"Descuento",
    en = u"Discount"
)

translations.define("Discount-plural",
    ca = u"Descomptes",
    es = u"Descuentos",
    en = u"Discounts"
)

# AbsoluteDiscount
#------------------------------------------------------------------------------
translations.define("PriceOverride",
    ca = u"Preu ajustat",
    es = u"Precio ajustado",
    en = u"Price override"
)

translations.define("PriceOverride-plural",
    ca = u"Preus ajustats",
    es = u"Precios ajustados",
    en = u"Price overrides"
)

translations.define("PriceOverride.price",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price"
)

# RelativeDiscount
#------------------------------------------------------------------------------
translations.define("RelativeDiscount",
    ca = u"Descompte relatiu",
    es = u"Descuento relativo",
    en = u"Relative discount"
)

translations.define("RelativeDiscount-plural",
    ca = u"Descomptes relatius",
    es = u"Descuentos relativos",
    en = u"Relative discounts"
)

translations.define("RelativeDiscount.amount",
    ca = u"Descompte",
    es = u"Descuento",
    en = u"Discount"
)

# PercentageDiscount
#------------------------------------------------------------------------------
translations.define("PercentageDiscount",
    ca = u"Descompte percentual",
    es = u"Descuento porcentual",
    en = u"Percentage discount"
)

translations.define("PercentageDiscount-plural",
    ca = u"Descomptes percentuals",
    es = u"Descuentos porcentuales",
    en = u"Percentage discounts"
)

translations.define("PercentageDiscount.percentage",
    ca = u"Percentatge",
    es = u"Porcentaje",
    en = u"Percentage"
)

# FreeUnitsDiscount
#------------------------------------------------------------------------------
translations.define("FreeUnitsDiscount",
    ca = u"Unitats gratuïtes",
    es = u"Unidades gratuitas",
    en = u"Free units"
)

translations.define("FreeUnitsDiscount-plural",
    ca = u"Unitats gratuïtes",
    es = u"Unidades gratuitas",
    en = u"Free units"
)

translations.define("FreeUnitsDiscount.paid_units",
    ca = u"Unitats pagades",
    es = u"Unidades pagadas",
    en = u"Paid units"
)

translations.define("FreeUnitsDiscount.free_units",
    ca = u"Unitats de regal",
    es = u"Unidades de regalo",
    en = u"Free units"
)

translations.define("FreeUnitsDiscount.repeated",
    ca = u"Admet múltiples",
    es = u"Admite múltiples",
    en = u"Repeated"
)

# ShippingCost
#------------------------------------------------------------------------------
translations.define("ShippingCost",
    ca = u"Cost d'enviament",
    es = u"Coste de envío",
    en = u"Shipping cost"
)

translations.define("ShippingCost-plural",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs"
)

# ShippingCostOverride
#------------------------------------------------------------------------------
translations.define("ShippingCostOverride",
    ca = u"Cost d'enviament absolut",
    es = u"Coste de envío absoluto",
    en = u"Shipping cost override"
)

translations.define("ShippingCostOverride-plural",
    ca = u"Costos d'enviament absoluts",
    es = u"Costes de envío absolutos",
    en = u"Shipping cost overrides"
)

translations.define("ShippingCostOverride.cost",
    ca = u"Cost",
    es = u"Coste",
    en = u"Cost"
)

# CumulativeShippingCost
#------------------------------------------------------------------------------
translations.define("CumulativeShippingCost",
    ca = u"Cost d'enviament acomulatiu",
    es = u"Coste de envío acomulativo",
    en = u"Cumulative shipping cost"
)

translations.define("CumulativeShippingCost-plural",
    ca = u"Costos d'enviament acomulatius",
    es = u"Costes de envío acomulativos",
    en = u"Cumulative shipping costs"
)

translations.define("CumulativeShippingCost.cost",
    ca = u"Cost",
    es = u"Coste",
    en = u"Cost"
)

# Tax
#------------------------------------------------------------------------------
translations.define("Tax",
    ca = u"Impost",
    es = u"Impuesto",
    en = u"Tax"
)

translations.define("Tax-plural",
    ca = u"Impostos",
    es = u"Impuestos",
    en = u"Taxes"
)

# CumulativeTax
#------------------------------------------------------------------------------
translations.define("CumulativeTax",
    ca = u"Impost acumulatiu",
    es = u"Impuesto acumulativo",
    en = u"Cumulative tax"
)

translations.define("CumulativeTax-plural",
    ca = u"Impostos acumulatius",
    es = u"Impuestos acumulativos",
    en = u"Cumulative taxes"
)

translations.define("CumulativeTax.cost",
    ca = u"Import",
    es = u"Importe",
    en = u"Cost"
)

# PercentageTax
#------------------------------------------------------------------------------
translations.define("PercentageTax",
    ca = u"Impost percentual",
    es = u"Impuesto porcentual",
    en = u"Percentage tax"
)

translations.define("PercentageTax-plural",
    ca = u"Impostos percentuals",
    es = u"Impuestos porcentuales",
    en = u"Percentage taxes"
)

translations.define("PercentageTax.percentage",
    ca = u"Percentatge",
    es = u"Porcentaje",
    en = u"Percentage"
)

# Views
#------------------------------------------------------------------------------
translations.define("woost.extensions.shop.ShopOrderTable product header",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product"
)

translations.define("woost.extensions.shop.ShopOrderTable quantity header",
    ca = u"Unitats",
    es = u"Unidades",
    en = u"Units"
)

translations.define("woost.extensions.shop.ShopOrderTable price header",
    ca = u"Preu",
    es = u"Precio",
    en = u"Price"
)

translations.define("woost.extensions.shop.ShopOrderTable total header",
    ca = u"Total",
    es = u"Total",
    en = u"Total"
)

translations.define("woost.extensions.shop.ShopOrderTable shipping costs",
    ca = u"Costos d'enviament",
    es = u"Costes de envío",
    en = u"Shipping costs"
)

translations.define("woost.extensions.shop.ShopOrderTable taxes",
    ca = u"Impostos",
    es = u"Impuestos",
    en = u"Taxes"
)

translations.define("woost.extensions.shop.ShopOrderTable total",
    ca = u"Total",
    es = u"Total",
    en = u"Total"
)

translations.define("woost.extensions.shop.ShoppingBasket remove entry",
    ca = u"Eliminar",
    es = u"Eliminar",
    en = u"Delete"
)

translations.define("woost.extensions.shop.ShoppingBasket update button",
    ca = u"Actualitzar preu",
    es = u"Actualizar precio",
    en = u"Update"
)

# UserFilter
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.shop.userfilter.ShopOrderCostFilter-instance",
    ca = u"Cost de la comanda",
    es = u"Coste del pedido ",
    en = u"Shop order cost"
)

translations.define("ShopOrderCostFilter.include_shipping",
    ca = u"Costos d'enviament inclosos",
    es = u"Costes de envío incluidos",
    en = u"Shipping costs included"
)

translations.define("ShopOrderCostFilter.include_discounts",
    ca = u"Descomptes inclosos",
    es = u"Descuentos incluidos",
    en = u"Discounts included"
)

translations.define("ShopOrderCostFilter.include_taxes",
    ca = u"Impostos inclosos",
    es = u"Impuestos incluidos",
    en = u"Taxes included"
)

# ShopController
#------------------------------------------------------------------------------
translations.define("woost.extensions.shop Product controller title",
    ca = u"Controlador de producte",
    es = u"Controlador de producto",
    en = u"Product controller"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Item


class OpenGraphCategory(Item):

    visible_from_root = False

    members_order = [
        "title",
        "code",
        "types"
    ]

    title = schema.String(
        required = True,
        unique = True,
        translated = True,
        descriptive = True
    )

    code = schema.String(
        required = True,
        unique = True,
        indexed = True
    )

    types = schema.Collection(
        items = "woost.extensions.opengraph.opengraphtype.OpenGraphType",
        bidirectional = True,
        cascade_delete = True
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cStringIO import StringIO
from subprocess import Popen, PIPE
from cocktail.translations import translations

p = Popen(["which", "dot"], stdout=PIPE)
graphviz_command = p.communicate()[0].replace("\n", "") or None
del p

def workflow_graph(states):
    """Renders the graph for the given workflow.

    :param states: The root states to include in the graph.
    :type states: `State`

    :return: DOT markup for the given workflow, to be rendered with Graphviz.
    :rtype: str
    """
    source = ["digraph workflow {"]
    source.append("\trankdir=LR;")
    source.append("\tnode [shape=ellipse];")
    source.append("\tedge [arrowsize=0.7];")

    def state_name(state):
        return "state%d" % state.id

    visited = set()

    def traverse(state):
        if state in visited:
            return

        visited.add(state)

        source.append(
            '\t%s [label="%s", id="%d"];'
            % (state_name(state), translations(state), state.id)
        )

        for transition in state.outgoing_transitions:
            if transition.target_state is not None:
                source.append('\t%s -> %s [label="%s", id="%d"];' % (
                    state_name(state),
                    state_name(transition.target_state),
                    transition.title,
                    transition.id
                ))
            traverse(transition.target_state)

    for state in states:
        traverse(state)

    source.append("}")
    return "\n".join(source)

def render_graph(markup, dest, format, command = graphviz_command):
    """Writes the specified DOT markup to the indicated file."""

    dest_is_path = isinstance(dest, basestring)

    cmd = [command, "-T", format]
    if dest_is_path:
        cmd.append("-o")
        cmd.append(dest)

    proc = Popen(
        cmd,
        stdin = PIPE,
        stdout = PIPE if not dest_is_path else None
    )
    stdout, stderr = proc.communicate(markup.encode("utf-8"))

    if not dest_is_path:
        dest.write(stdout)


if __name__ == "__main__":

    import sys
    import os
    from cocktail.translations import set_language
    from woost.extensions.workflow.state import State
    import testproject.models

    dest = sys.stdout if len(sys.argv) < 2 else sys.argv[1]
    format = None

    if len(sys.argv) >= 3:
        format = sys.argv[2]
    elif isinstance(dest, basestring):
        format = os.path.splitext(dest)[1]
        if format:
            format = format[1:]

    if not format:
        format = "svg"

    set_language("ca")
    working = State.require_instance(qname = "testproject.working_state")
    graph = workflow_graph(working)
    render_graph(graph, dest, format)




#-*- coding: utf-8 -*-
u"""
Translated strings for the Woost CMS.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from woost.translations import strings




#-*- coding: utf-8 -*-
"""
Selenium test suite for the Woost CMS.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from os.path import join
from shutil import rmtree
from tempfile import mkdtemp
from ZODB.FileStorage import FileStorage
import cherrypy
from cocktail.persistence import datastore
from cocktail.tests.seleniumtester import (
    get_selenium_enabled,
    get_selenium_site_address,
    browser
)
from woost import app
from woost.models.initialization import init_site
from woost.controllers.cmscontroller import CMSController

# Site defaults
admin_email = "admin@localhost"
admin_password = "topsecret"
site_languages = ("en",)

# Path for site temporary files
_site_temp_path = None

def login(user, password):
    browser.type("user", user)
    browser.type("password", password)
    browser.click("authenticate")
    browser.wait_for_page_to_load(10000)

def admin_login():
    login(admin_email, admin_password)


def setup_package():

    if not get_selenium_enabled():
        return

    # Create a temporary folder to hold site files
    _site_temp_path= mkdtemp()
    app.root = _site_temp_path

    # Set up a temporary database
    datastore.storage = FileStorage(app.path("testdb.fs"))

    # Initialize site content before testing
    init_site(
        admin_email,
        admin_password,
        site_languages
    )
    datastore.commit()

    # Configure the site's webserver
    hostname, port = get_selenium_site_address()
    cherrypy.config.update({
        "log.screen": False,
        "server.socket_host": hostname,
        "server.socket_port": port,
        "engine.autoreload.on": False
    })

    # Configure the application
    cms = CMSController()
    cms.closing_item_requires_confirmation = False

    # Launch the site's webserver on another thread
    cms.run(block = False)

def teardown_package():

    if not get_selenium_enabled():
        return

    # Stop the site's webserver
    cherrypy.server.stop()
    cherrypy.engine.exit()

    # Remove temporary site files:
    # This will often fail on Windows, probably because we try to delete a
    # running executable, so delay & retry a limited number of times, to give
    # any running process a chance to end.
    for i in range(5):
        try:
            rmtree(_site_temp_path)
        except Exception:
            from time import sleep
            sleep(1)
        else:
            break




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import sys
import os
from optparse import OptionParser
from PyQt4.QtCore import Qt, QObject, QUrl, QSize, SIGNAL
from PyQt4.QtGui import QApplication, QPainter, QPixmap
from PyQt4.QtWebKit import QWebView

_formats_by_extension = {
    ".png": "PNG",
    ".bmp": "BMP",
    ".xbm": "XBM",
    ".xpm": "XPM",
    ".jpg": "JPEG",
    ".jpeg": "JPEG",
    ".gif": "GIF"
}

DEFAULT_QUALITY = -1
DEFAULT_MIN_WIDTH = 800
DEFAULT_MIN_HEIGHT = 600

def render_url(url, dest, quality = -1, min_width = 800, min_height = 600):

    format = get_image_format(dest)

    if format is None:
        raise RuntimeError("Can't handle the specified image format")

    app = QApplication(sys.argv)
    web = QWebView()
    web.load(QUrl(url))

    def save_image():

        try:
            page = web.page()
            frame = page.currentFrame()

            # Grow the viewport to make it fit the whole document
            width = min_width
            height = min_height

            page.setViewportSize(QSize(width, height))

            while frame.scrollBarMaximum(Qt.Horizontal) != 0:
                width += 1
                page.setViewportSize(QSize(width, height))

            while frame.scrollBarMaximum(Qt.Vertical) != 0:
                height += 1
                page.setViewportSize(QSize(width, height))

            # Render the document to an image file
            image = QPixmap(width, height)
            painter = QPainter()
            painter.begin(image)
            frame.render(painter)
            painter.end()
            image.save(dest, format, quality)

        finally:
            QApplication.exit()

    QObject.connect(web, SIGNAL("loadFinished(bool)"), save_image)
    app.exec_()

def get_image_format(file_name):
    return _formats_by_extension.get(os.path.splitext(file_name)[1])

def main():

    parser = OptionParser()

    parser.add_option("--quality",
        help = u"The quality for the image")

    parser.add_option("--min-width",
        help = u"The minimum width for the viewport")

    parser.add_option("--min-height",
        help = u"The minimum height for the viewport")

    options, args = parser.parse_args()

    if len(args) != 2:
        sys.stderr.write("Usage: %s URL DEST\n" % sys.argv[0])
        sys.exit(1)

    kwargs = {}

    if options.quality:
        kwargs["quality"] = int(options.quality)

    if options.min_width:
        kwargs["min_width"] = int(options.min_width)

    if options.min_height:
        kwargs["min_height"] = int(options.min_height)

    render_url(*args, **kwargs)

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import sys
import socket
import re
import os
from shutil import rmtree
from subprocess import Popen, PIPE
from pkg_resources import resource_filename
import buffet
import cherrypy
from beaker.middleware import SessionMiddleware
from cocktail import schema
from cocktail.translations import set_language
from cocktail.html import templates
from cocktail.controllers import get_parameter
from woost.translations import installerstrings
from woost.models.initialization import init_site


class Installer(object):

    base_path = os.path.abspath(resource_filename("woost", ""))
    skeleton_path = os.path.join(base_path, "scripts","project_skeleton")
    views_path = os.path.join(base_path, "views")

    resources = cherrypy.tools.staticdir.handler(
        section = "resources",
        root = views_path,
        dir = "resources"
    )

    @cherrypy.expose
    def index(self, **params):

        set_language("en")
        submitted = cherrypy.request.method == "POST"
        successful = False
        errors = []

        HOST_FORMAT = \
            re.compile(r"^([a-z]+(\.[a-z]+)*)|(\d{1,3}(\.\d{1,3}){3})$")

        paths = sorted([p for p in sys.path if os.access(p, os.W_OK)])

        form_schema = schema.Schema(
            name = "Installer",
            members = [
                schema.String(
                    name = "project_name",
                    required = True,
                    format = r"^[A-Z][A-Za-z_0-9]*$",
                    member_group = "project"
                ),
                schema.String(
                    name = "python_package_repository",
                    required = True,
                    enumeration = paths,
                    default = paths and paths[0] or None,
                    member_group = "project"
                ),
                schema.String(
                    name = "admin_email",
                    required = True,
                    default = "admin@localhost",
                    member_group = "project"
                ),
                schema.String(
                    name = "admin_password",
                    required = True,
                    min = 8,
                    edit_control = "cocktail.html.PasswordBox",
                    member_group = "project"
                ),
                schema.String(
                    name = "languages",
                    required = True,
                    default = "en",
                    format = r"^[a-zA-Z]{2}(\W+[a-zA-Z]{2})*$",
                    member_group = "project"
                ),
                schema.String(
                    name = "template_engine",
                    required = True,
                    default = "cocktail",
                    enumeration = buffet.available_engines.keys(),
                    member_group = "project"
                ),
                schema.String(
                    name = "webserver_host",
                    required = True,
                    format = HOST_FORMAT,
                    default = "127.0.0.1",
                    member_group = "webserver"
                ),
                schema.Integer(
                    name = "webserver_port",
                    required = True,
                    default = 8080,
                    member_group = "webserver"
                ),
                schema.Boolean(
                    name = "validate_webserver_address",
                    default = True,
                    member_group = "webserver"
                ),
                schema.String(
                    name = "database_host",
                    required = True,
                    format = HOST_FORMAT,
                    default = "127.0.0.1",
                    member_group = "database"
                ),
                schema.Integer(
                    name = "database_port",
                    required = True,
                    default = 8081,
                    member_group = "database"
                ),
                schema.Boolean(
                    name = "validate_database_address",
                    default = True,
                    member_group = "database"
                )
            ]
        )

        def make_address_validation(host_field, port_field, check_field):

            def validate_address(schema, validable, context):

                if validable[check_field]:
                    host = validable[host_field]
                    port = validable[port_field]
                    host_member = schema[host_field]
                    port_member = schema[port_field]

                    if host_member.validate(host) \
                    and port_member.validate(port) \
                    and not self._is_valid_local_address(host, port):
                        yield WrongAddressError(
                            schema, validable, context,
                            host_member, port_member
                        )

            return validate_address

        form_schema.add_validation(make_address_validation(
            "webserver_host",
            "webserver_port",
            "validate_webserver_address"
        ))

        form_schema.add_validation(make_address_validation(
            "database_host",
            "database_port",
            "validate_database_address"
        ))

        form_data = {}

        if submitted:
            get_parameter(
                form_schema,
                target = form_data,
                errors = "ignore",
                undefined = "set_none"
            )
            errors = list(form_schema.get_errors(form_data))

            if not errors:

                form_data["project_path"] = os.path.join(
                    form_data["python_package_repository"],
                    form_data["project_name"].lower()
                )

                try:
                    if os.path.exists(form_data["project_path"]):
                        raise InstallFolderExists()

                    self.install(form_data)

                except Exception, ex:
                    errors.append(ex)
                    if not isinstance(ex, InstallFolderExists):
                        try:
                            rmtree(form_data["project_path"])
                        except Exception, rmex:
                            errors.append(rmex)
                else:
                    successful = True
        else:
            form_schema.init_instance(form_data)

        view = templates.new("woost.views.Installer")
        view.submitted = submitted
        view.successful = successful
        view.schema = form_schema
        view.data = form_data
        view.errors = errors
        return view.render_page()

    def install(self, params):

        params["project_module"] = params["project_name"].lower()

        self._create_project(params)
        self._init_project(params)

    def _create_project(self, params):

        vars = dict(
            ("_%s_" % key.upper(), unicode(value))
            for key, value in params.iteritems()
        )

        keys_expr = re.compile("|".join(vars))

        def expand_vars(text):
            return keys_expr.sub(lambda match: vars[match.group(0)], text)

        def copy(source, target):

            if os.path.isdir(source):
                os.mkdir(target)
                for name in os.listdir(source):
                    copy(
                        os.path.join(source, name),
                        os.path.join(target, expand_vars(name))
                    )

            elif os.path.isfile(source):
                if os.path.splitext(source)[1] != ".pyc":
                    source_file = file(source, "r")
                    try:
                        source_data = source_file.read().decode("utf-8")
                        target_data = expand_vars(source_data).encode("utf-8")
                        target_file = file(target, "w")
                        try:
                            target_file.write(target_data)
                        finally:
                            target_file.close()
                    finally:
                        source_file.close()

        copy(self.skeleton_path, params["project_path"])

        # Create the folder for the database
        os.mkdir(os.path.join(params["project_path"], "data"))

        # Grant execution permission for project scripts
        scripts_path = os.path.join(params["project_path"], "scripts")

        for fname in os.listdir(scripts_path):
            if fname != "__init__.py":
                script = os.path.join(scripts_path, fname)
                if os.path.isfile(script):
                    os.chmod(script, 0774)

    def _subprocess(self, cmd):
        # - Not used -
        proc = Popen(cmd, shell = True, stderr = PIPE)
        if proc.wait() != 0:
            raise SubprocessError(cmd, proc.stderr.read())

    def _init_project(self, params):

        # Start the database
        cmd = "runzeo -f %s -a %s:%d" % (
            os.path.join(params["project_path"], "data", "database.fs"),
            params["database_host"],
            params["database_port"]
        )
        Popen(cmd, shell = True)

        __import__(params["project_module"])
        init_site(
            admin_email = params["admin_email"],
            admin_password = params["admin_password"],
            languages = params["languages"].split(),
            template_engine = params["template_engine"]
        )

    def _is_valid_local_address(self, host, port):
        s = socket.socket()
        try:
            s.bind((host, port))
        except socket.error:
            return False
        else:
            s.close()

        return True

    def run(self):
        cherrypy.quickstart(self, "/", config = self.get_configuration())

    def get_configuration(self):
        return {
            "global": {
                "server.socket_port": 10000,
                "tools.encode.on": True,
                "tools.encode.encoding": "utf-8",
                "tools.decode.on": True,
                "tools.decode.encoding": 'utf-8',
                "engine.autoreload_on": False
            },
            "/": {
                "wsgi.pipeline": [("beaker", SessionMiddleware)],
                "wsgi.beaker.config": {"session.type": "memory"}
            }
        }


class InstallFolderExists(Exception):
    pass


class WrongAddressError(schema.exceptions.ValidationError):

    def __init__(self, member, value, context, host_member, port_member):

        schema.exceptions.ValidationError.__init__(
            self, member, value, context
        )

        self.host_member = host_member
        self.port_member = port_member


class PythonPathError(schema.exceptions.ValidationError):
    """A validation error produced when trying to install a project on a path
    that lies outside the python path."""


class SubprocessError(Exception):

    def __init__(self, cmd, error_output):
        self.cmd = cmd
        self.error_output = error_output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from cocktail import schema
from woost.models.document import Document


class Event(Document):

    members_order = [
        "event_start",
        "event_end",
        "event_location",
        "body"
    ]

    event_start = schema.DateTime(
        member_group = "content"
    )

    event_end = schema.DateTime(
        member_group = "content",
        min = event_start
    )

    event_location = schema.String(
        edit_control = "cocktail.html.TextArea",
        translated = True,
        member_group = "content"
    )

    body = schema.String(
        edit_control = "woost.views.RichTextEditor",
        translated = True,
        member_group = "content"
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from woost.models import (
    CreatePermission,
    ModifyPermission,
    DeletePermission
)
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.vimeo.video import VimeoVideo


class SynchronizeVimeoAction(UserAction):

    ignores_selection = True
    min = None
    max = None
    included = frozenset(["toolbar"])
    excluded = frozenset(["collection"])
    content_type = VimeoVideo

    def is_permitted(self, user, target):

        # Must be able to create, update or delete videos (any of those
        # permissions will enable the button; the controller will later
        # limit which actions are actually allowed)
        return any(
            user.has_permission(permission_type, target = target)
            for permission_type in (
                CreatePermission,
                ModifyPermission,
                DeletePermission
            )
        )

SynchronizeVimeoAction("sync_vimeo").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block
from woost.models.rendering import ImageFactoryMember


class FolderBlock(Block):

    instantiable = True
    view_class = "woost.extensions.blocks.FolderBlockView"

    members_order = [
        "show_hidden_children",
        "show_thumbnails",
        "thumbnails_factory"
    ]

    show_hidden_children = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    show_thumbnails = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    thumbnails_factory = ImageFactoryMember(
        exclusive = show_thumbnails,
        default = False,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.show_hidden_children = self.show_hidden_children
        view.show_thumbnails = self.show_thumbnails
        view.thumbnails_factory = self.thumbnails_factory




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Publishable, Controller


class TextFile(Publishable):

    instantiable = True
    per_language_publication = False
    default_mime_type = "text/plain"
    default_hidden = True

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(
            qname = "woost.extensions.textfile.TextFileController"
        )
    )

    members_order = ["title", "content"]

    title = schema.String(
        translated = True,
        descriptive = True,
        listed_by_default = False,
        member_group = "content"
    )

    content = schema.String(
        edit_control = "cocktail.html.TextArea",
        listed_by_default = False,
        member_group = "content"
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import cherrypy
from woost.models import Style
from woost.controllers.publishablecontroller import PublishableController


class StylesController(PublishableController):

    def _produce_content(self, backoffice = False):

        for style in Style.select():
            declarations = (backoffice and style.admin_declarations) or style.declarations
            yield ".%s{\n%s\n}\n" % (style.class_name, declarations)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.schema.exceptions import ValidationError
from woost import app
from woost.models import Publishable

def requires_agreement(form, name = "terms", document = None):

    if document is None:
        document = "%s.%s" % (app.package, name)

    if isinstance(document, basestring):
        document = Publishable.require_instance(qname = document)

    if document is None:
        raise ValueError(
            "Must specify a document detailing the end user agreement"
        )

    member = schema.Boolean(
        name = name,
        required = True,
        __translate__ = lambda language, **kwargs:
            schema.Boolean.__translate__(member, language, **kwargs)
            or translations(
                "woost.controllers.formagreement.%s" % member.name,
                member = member
            )
            or translations("woost.controllers.formagreement", member = member)
    )
    member.agreement_document = document
    member.add_validation(_validate_consent)

    form.schema.add_member(member)
    form.adapter.exclude(name)
    return member

def _validate_consent(member, value, ctx):
    if not value:
        yield ConsentNotGivenError(member, value, ctx)


class ConsentNotGivenError(ValidationError):
    """A validation error produced by forms that enforce an agreement to a
    terms and conditions document when the user attempts to submit the form
    without acknowledging the agreement.
    """




#-*- coding: utf-8 -*-
u"""
Provides a member that handles reCAPTCHA values.

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
import urllib
import urllib2
import cherrypy
from cocktail.schema.schema import Schema
from cocktail.schema.schemastrings import String
from cocktail.schema.accessors import get_accessor
from cocktail.schema.exceptions import ValidationError
from woost.extensions.recaptcha import ReCaptchaExtension


class ReCaptcha(Schema):
    """A member that handles reCAPTCHA values.
    """

    VERIFY_SERVER = "api-verify.recaptcha.net"

    def __init__(self, *args, **kwargs):
        Schema.__init__(self, *args, **kwargs)

        def read_challenge(reader):
            return reader.source("recaptcha_challenge_field")

        def read_response(reader):
            return reader.source("recaptcha_response_field")

        self.add_member(String(
            "challenge",
            read_request_value = read_challenge
        ))
        self.add_member(String(
            "response",
            read_request_value = read_response
        ))
        self.add_validation(ReCaptcha.captcha_validation_rule)

        self.edit_control = "woost.extensions.recaptcha.ReCaptchaBox"

    def captcha_validation_rule(self, validable, context):
        """Validation rule for reCaptcha. Applies the validation rules defined
        by all members in the schema, propagating their errors. Checks that the
        L{response} member is valid for the L{challenge} and the L{public_key}
        constraint.
        """

        accessor = self.accessor \
            or context.get("accessor", None) \
            or get_accessor(validable)

        challenge = accessor.get(validable, "challenge", default = None)
        response = accessor.get(validable, "response", default = None)

        if challenge and response:

            def encode_if_necessary(s):
                if isinstance(s, unicode):
                    return s.encode('utf-8')
                return s

            private_key = ReCaptchaExtension.instance.private_key
            remote_ip = cherrypy.request.remote.ip

            params = urllib.urlencode({
                'privatekey': encode_if_necessary(private_key),
                'remoteip': encode_if_necessary(remote_ip),
                'challenge': encode_if_necessary(challenge),
                'response': encode_if_necessary(response),
            })

            request = urllib2.Request(
                url = "http://%s/verify" % self.VERIFY_SERVER,
                data = params,
                headers = {
                    "Content-type": "application/x-www-form-urlencoded",
                    "User-agent": "Woost reCAPTCHA extension"
                }
            )

            httpresp = urllib2.urlopen(request)

            return_values = httpresp.read().splitlines();
            httpresp.close();

            return_code = return_values[0]

            if (return_code != "true"):
                yield ReCaptchaValidationError(self, response, context)

        else:
            yield ReCaptchaValidationError(self, response, context)

    def _create_default_instance(self):
        return None


class ReCaptchaValidationError(ValidationError):
    """A validation error produced when the user fails a ReCaptcha
    validation.
    """




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension


translations.define("TwitterPublicationExtension",
    ca = u"Publicació a Twitter",
    es = u"Publicación en Twitter",
    en = u"Publish to Twitter"
)

translations.define("TwitterPublicationExtension-plural",
    ca = u"Publicació a Twitter",
    es = u"Publicación en Twitter",
    en = u"Publish to Twitter"
)


class TwitterPublicationExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Publica continguts del lloc web al Twitter""",
            "ca"
        )
        self.set("description",
            u"""Publica contenidos del sitio web en Twitter""",
            "es"
        )
        self.set("description",
            u"""Publish content from the website to Twitter""",
            "en"
        )

    def _load(self):

        from woost.extensions.twitterpublication import (
            strings,
            tweetuseraction,
            twitterauthuseraction,
            tweetpermission
        )

        from woost.extensions.twitterpublication.twitterpublicationtarget \
            import TwitterPublicationTarget

        TwitterPublicationExtension.add_member(
            schema.Collection("targets",
                items = schema.Reference(type = TwitterPublicationTarget),
                integral = True,
                bidirectional = True,
                related_end = schema.Collection()
            )
        )

        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController
        from woost.extensions.twitterpublication.tweetcontroller \
            import TweetController

        BackOfficeController.tweet = TweetController

        self.install()

    def _install(self):

        from woost.models import Role
        from woost.extensions.twitterpublication.tweetpermission \
            import TweetPermission

        # By default, only administrator users can tweet content
        permission = TweetPermission()
        permission.matching_items = {
            "type": "woost.models.publishable.Publishable"
        }
        permission.insert()

        admins = Role.require_instance(qname = "woost.administrators")
        admins.permissions.append(permission)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from os.path import abspath, dirname, join
from urlparse import urljoin
from cocktail.html import templates, Element
from cocktail.translations import translations
from cocktail.controllers import context
from woost.models import Style

TinyMCE = templates.get_class("cocktail.html.TinyMCE")

#Required Version: 3.2.2.3 TinyMCE
class RichTextEditor(TinyMCE):

    tinymce_params = {
        "plugins": "fullscreen, paste, media, inlinepopups, advimage, "
                   "contextmenu, tabfocus, -advimagescale",
        "entity_encoding": "raw",
        "dialog_type": "modal",
        "theme": "advanced",
        "theme_advanced_buttons1_add": "removeformat",
        "theme_advanced_buttons2_add": "selectall, | , fullscreen",
        "theme_advanced_buttons3": "",
        "theme_advanced_toolbar_location": "top",
        "theme_advanced_resizing": True,
        "theme_advanced_statusbar_location": "bottom",
        "theme_advanced_toolbar_align": "left",
        "theme_advanced_path": False,
        "theme_advanced_resize_horizontal": False,
        "document_base_url": "/",
        "relative_urls": False,
        "content_css": "/user_styles/?backoffice=1",
        "fullscreen_settings": {
            "theme_advanced_toolbar_location": "top"
        },
        "height": 250,
        "media_strict": False,
        "paste_text_sticky_default": True
    }

    def _ready(self):

        ctx_uri = context["cms"].contextual_uri()
        current_edit_stack = \
            context["edit_stacks_manager"].current_edit_stack

        styles = [
            "%s=%s" % (translations(style), style.class_name)
            for style in Style.select()
        ]

        if current_edit_stack:
            edit_stack_param = current_edit_stack.to_param()
            attachments_uri = ctx_uri.rstrip("/") + \
                "/editor_attachments?edit_stack=%s&language=%s&resource_type=" % (
                    edit_stack_param, self.language
                )

            d = self.tinymce_params.setdefault
            d("theme_advanced_styles", ";".join(styles))
            d("init_instance_callback", "woost.initRichTextEditor")
            d("external_image_list_url", attachments_uri + "image")
            d("external_link_list_url", attachments_uri + "linkable")
            d("media_external_list_url", attachments_uri + "video")

        load_plugin = Element("script")
        load_plugin["type"] = "text/javascript"
        load_plugin.append("tinymce.PluginManager.load('advimagescale', '/resources/scripts/advimagescale/editor_plugin.js');")
        self.append(load_plugin)

        TinyMCE._ready(self)

    def __init__(self, *args, **kwargs):
        TinyMCE.__init__(self, *args, **kwargs)
        self.add_resource("/resources/scripts/RichTextEditor.js")




#-*- coding: utf-8 -*-
u"""Defines the `TweetPermission` model.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models.permission import (
    ContentPermission,
    permission_doesnt_match_style
)
from woost.extensions.twitterpublication.twitterpublicationtarget \
    import TwitterPublicationTarget


class TweetPermission(ContentPermission):

    instantiable = True

    publication_targets = schema.Collection(
        items = schema.Reference(type = TwitterPublicationTarget),
        related_end = schema.Collection(),
        edit_inline = True,
        after_member = "matching_items"
    )

    def match(self, target, publication_target = None, verbose = False):

        if not ContentPermission.match(self, target, verbose = verbose):
            return False

        if publication_target is not None \
        and self.publication_targets \
        and publication_target not in self.publication_targets:
            print permission_doesnt_match_style(
                "publication target doesn't match"
            ),

        return True




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension

translations.define("GoogleAnalyticsExtension",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension-plural",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension.account",
    ca = u"Compte",
    es = u"Cuenta",
    en = u"Account"
)


class GoogleAnalyticsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Integra el lloc web amb Google Analytics.""",
            "ca"
        )
        self.set("description",
            u"""Integra el sitio web con Google Analytics.""",
            "es"
        )
        self.set("description",
            u"""Integrates the site with Google Analytics.""",
            "en"
        )

    account = schema.String(
        required = True,
        text_search = False
    )

    def _load(self):
        from cocktail.events import when
        from woost.controllers import CMSController
        from woost.extensions.googleanalytics import (
            strings,
            document
        )

        @when(CMSController.producing_output)
        def handle_producing_output(e):
            publishable = e.output.get("publishable")
            if (
                publishable is None
                or getattr(publishable, "is_ga_tracking_enabled", lambda: True)()
            ):
                html = e.output.get("head_end_html", "")
                if html:
                    html += " "
                html += self.get_analytics_script()
                e.output["head_end_html"] = html

    def get_analytics_script(self):
        return """
        <script type="text/javascript">
            var _gaq = _gaq || [];
            _gaq.push(['_setAccount', '%s']);
            _gaq.push(['_trackPageview']);
            (function() {
            var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
            ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
            var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
            })();
        </script>""" % self.account




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from cocktail import schema
from woost.models.document import Document
from woost.models.template import Template


class StandardPage(Document):

    default_template = schema.DynamicDefault(
        lambda: Template.get_instance(qname = "woost.standard_template")
    )

    body = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "woost.views.RichTextEditor",
        member_group = "content"
    )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from __future__ import with_statement
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import get_parameter, context as controller_context
from cocktail.controllers.viewstate import view_state
from cocktail.html import Element
from woost.models import get_current_user
from woost.models.changesets import changeset_context
from woost.controllers import notify_user
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.workflow.transitionpermission import \
    TransitionPermission
from woost.extensions.workflow.transition import Transition
from woost.extensions.workflow.transitioncontroller import \
    redirect_transition


class TransitionAction(UserAction):

    included = frozenset(["item_buttons"])
    excluded = frozenset(["new_item"])

    def is_available(self, context, target):

        # Hide the transition action unless there are one or more available
        # outgoing states for the item's current condition
        return UserAction.is_available(self, context, target) \
            and bool(self._get_outgoing_transitions(target))

    def get_dropdown_panel(self, item):

        panel = Element()

        for transition in self._get_outgoing_transitions(item):

            button = Element("a")

            # Transitions with parameters: redirect to the transition form
            if transition.transition_form:
                button["href"] = controller_context["cms"].contextual_uri(
                    "workflow_transition",
                    item = item.id,
                    transition = transition.id
                )

            # This should really be a POST operation, but HTML doesn't provide
            # multi-value buttons, so regular links and GET requests are all
            # that is left :(
            else:
                button["href"] = u"?" + view_state(
                    transition = transition.id,
                    item_action = "transition"
                )

            button.append(translations(transition))
            panel.append(button)

        return panel

    def _get_outgoing_transitions(self, item, restricted = True):

        if item.workflow_state is None:
            transitions = []
        else:
            transitions = item.workflow_state.outgoing_transitions

        if restricted:
            user = get_current_user()
            transitions = [
                transition
                for transition in transitions
                if user.has_permission(
                    TransitionPermission,
                    target = item,
                    transition = transition
                )
            ]

        return transitions

    def invoke(self, controller, selection):

        item = selection[0]
        draft_source = item.draft_source

        # Retrieve and validate the desired transition for the item
        transition = get_parameter(
            schema.Reference(
                "transition",
                type = Transition,
                enumeration = self._get_outgoing_transitions(
                    item,
                    # This allows to discriminate between invalid ids and authz
                    # errors:
                    restricted = False
                )
            )
        )

        if transition is None:
            raise ValueError("Wrong transition identifier")

        # Authorization check
        user = get_current_user()
        user.require_permission(
            TransitionPermission,
            target = item,
            transition = transition
        )

        # Transition the item to its new state
        with changeset_context(user):
            transition.execute(item)
            datastore.commit()

        # Inform the user of the result
        notify_user(
            translations(
                "woost.controllers.backoffice.useractions.TransitionAction"
                " state set",
                item = item
            ),
            "success"
        )

        redirect_transition(item if item.is_inserted else draft_source)

TransitionAction("transition").register(before = "close")




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define("Document.ga_tracking_enabled",
    ca = u"Habilitar seguiment amb Google Analytics",
    es = u"Habilitar seguimiento con Google Analytics",
    en = u"Track with Google Analytics"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import (
    request_property,
    FormProcessor
)
from woost.models import Publishable
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.ecommerce.basket import Basket
from woost.extensions.ecommerce.orderstepcontroller import (
    ProceedForm,
    BackForm
)
from woost.extensions.payments import PaymentsExtension


class SummaryController(FormProcessor, DocumentController):

    @request_property
    def checkout_schema(self):
        return Basket.get().get_public_schema()

    @request_property
    def output(self):
        output = DocumentController.output(self)
        output["checkout_schema"] = self.checkout_schema
        return output

    class SubmitOrderForm(ProceedForm):

        def submit(self):

            ProceedForm.submit(self)

            order = Basket.pop()
            order.status = "payment_pending"
            order.update_cost()

            datastore.commit()
            payments = PaymentsExtension.instance

            # Redirect the user to the payment gateway
            if payments.enabled \
            and payments.payment_gateway \
            and order.payment_type == "payment_gateway":
                payments.payment_request(order.id)

            # No payment gateway available, redirect the user to the success
            # page; the payment will have to be handled manually by the site's
            # personnel
            else:
                raise cherrypy.HTTPRedirect(
                    Publishable.require_instance(
                        qname = "woost.extensions.ecommerce.success_page"
                    ).get_uri(
                        parameters = {"order": order.id}
                    )
                )

    class PreviousStepForm(BackForm):
        pass




#-*- coding: utf-8 -*-
u"""
Provides base and default content types for the woost CMS.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from cocktail import schema
from cocktail.events import when

# Add an extension property to determine wether collections should be edited on
# the main tab of the object editor (True) or if they should be promoted to
# their own tab (False, default)
def _get_edit_inline(self):

    if self._edit_inline is None:
        return not self.is_persistent_relation

    return self._edit_inline

def _set_edit_inline(self, value):
    self._edit_inline = value

schema.Collection._edit_inline = None
schema.Collection.edit_inline = property(_get_edit_inline, _set_edit_inline)

# Add an extension property to control the default member visibility on item listings
schema.Member.listed_by_default = True
schema.Collection.listed_by_default = False
schema.CodeBlock.listed_by_default = False

# Add an extension property to indicate if members should be visible by users
schema.Member.visible = True

# Add an extension property to indicate if schemas should be instantiable by
# users
schema.Schema.instantiable = True

# Add an extension property to indicate if members should be editable by users
schema.Member.editable = True

# Add an extesnion property to indiciate if members should be shown in detailed view
schema.Member.visible_in_detail_view = True

# Add an extension property to indicate if relations should be excluded if no
# relatable elements exist
schema.Collection.exclude_when_empty = False

# Add an extension property to determine if members should participate in item
# revisions
schema.Member.versioned = True

# Add an extension property to allow relations to block a delete operation if
# the relation is not empty
schema.RelationMember.block_delete = False

@when(schema.RelationMember.attached_as_orphan)
def _hide_self_contained_relations(event):
    if event.anonymous:
        event.source.visible = False

# Register the 'text/javascript' MIME type
import mimetypes
if not mimetypes.guess_extension("text/javascript"):
    mimetypes.add_type("text/javascript", ".js")
del mimetypes

# Base content types
#------------------------------------------------------------------------------
from woost.models.site import Site
from woost.models.changesets import ChangeSet, Change, changeset_context
from woost.models.item import Item
from woost.models.action import Action
from woost.models.language import Language
from woost.models.userview import UserView
from woost.models.publicationschemes import (
    PublicationScheme,
    PathResolution,
    HierarchicalPublicationScheme,
    IdPublicationScheme,
    DescriptiveIdPublicationScheme
)
from woost.models.publishable import (
    Publishable,
    IsPublishedExpression,
    IsAccessibleExpression
)
from woost.models.document import Document
from woost.models.template import Template
from woost.models.controller import Controller
from woost.models.user import (
    User,
    AuthorizationError
)
from woost.models.usersession import (
    get_current_user,
    set_current_user
)
from woost.models.role import Role
from woost.models.permission import (
    Permission,
    CreatePermission,
    ReadPermission,
    ModifyPermission,
    DeletePermission,
    ConfirmDraftPermission,
    RenderPermission,
    ReadMemberPermission,
    ModifyMemberPermission,
    CreateTranslationPermission,
    ReadTranslationPermission,
    ModifyTranslationPermission,
    DeleteTranslationPermission,
    ReadHistoryPermission,
    restricted_modification_context,
    delete_validating,
    PermissionExpression,
    ChangeSetPermissionExpression
)
from woost.models.standardpage import StandardPage
from woost.models.file import File
from woost.models.news import News
from woost.models.event import Event
from woost.models.uri import URI
from woost.models.file import File
from woost.models.style import Style
from woost.models.extension import (
    Extension,
    extension_translations,
    load_extensions
)
from woost.models.trigger import (
    Trigger,
    ContentTrigger,
    CreateTrigger,
    InsertTrigger,
    ModifyTrigger,
    DeleteTrigger,
    ConfirmDraftTrigger
)
from woost.models.triggerresponse import (
    TriggerResponse,
    CustomTriggerResponse,
    SendEmailTriggerResponse
)
from woost.models.emailtemplate import EmailTemplate
from woost.models.feed import Feed

from woost.models.userfilter import (
    OwnItemsFilter,
    IsPublishedFilter,
    TypeFilter
)

from woost.models.caching import CachingPolicy, expire_cache

from woost.models import rendering
from woost.models import staticpublication
from woost.models import migration




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			Aug 2009
"""
from cocktail.events import when
from cocktail import schema
from cocktail.translations import translations
from cocktail.html import templates
from woost.models import Item
from woost.extensions.workflow.state import State
from woost.extensions.workflow.transition import Transition
from woost.models.messagestyles import trigger_doesnt_match_style
from woost.models.trigger import (
    ContentTrigger,
    members_without_triggers,
    trigger_responses
)

members_without_triggers.add(Item.workflow_state)


class TransitionTrigger(ContentTrigger):

    instantiable = True

    def _transition_edit_control(parent, obj, member):
        display = templates.new("cocktail.html.DropdownSelector")
        display.empty_label = translations(
            "woost.extensions.workflow.TransitionTrigger any transition"
        )
        return display

    transition = schema.Reference(
        type = "woost.extensions.workflow.transition.Transition",
        edit_control = _transition_edit_control
    )

    del _transition_edit_control

    def match(
        self,
        user,
        target,
        transition,
        transition_data,
        verbose = False,
        **context):

        if self.transition is not None and transition is not self.transition:
            if verbose:
                print trigger_doesnt_match_style("transition doesn't match")
            return False

        return ContentTrigger.match(self,
            user,
            target = target,
            transition = transition,
            transition_data = transition_data,
            verbose = verbose,
            **context
        )


@when(Transition.executed)
def _trigger_transition_responses(event):
    trigger_responses(
        TransitionTrigger,
        target = event.item,
        transition = event.source,
        transition_data = event.data
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block
from woost.extensions.vimeo.video import VimeoVideo


class VimeoBlock(Block):

    instantiable = True
    view_class = "woost.extensions.vimeo.VimeoPlayer"

    members_order = "video", "video_width", "video_height"


    video = schema.Reference(
        required = True,
        type = VimeoVideo,
        related_end = schema.Collection(),
        member_group = "content",
    )

    video_width = schema.Integer(
        member_group = "content"
    )

    video_height = schema.Integer(
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.video = self.video
        if self.video_width:
            view.flash_width = self.video_width
        if self.video_height:
            view.flash_height = self.video_height




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import Publishable
from woost.extensions.blocks.block import Block


class RichTextBlock(Block):

    instantiable = True
    view_class = "cocktail.html.Element"

    text = schema.String(
        required = True,
        edit_control = "woost.views.RichTextEditor",
        member_group = "content"
    )

    attachments = schema.Collection(
        items = schema.Reference(type = Publishable),
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.append(self.text)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.controllers import Controller


class PaymentHandshakeController(Controller):

    def __call__(self, **parameters):
        from woost.extensions.payments import PaymentsExtension
        gateway = PaymentsExtension.instance.payment_gateway
        return gateway.handle_handshake(parameters)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.modeling import getter
from cocktail.html import templates
from woost.models import File
from woost.models.rendering.factories import ImageFactoryMember
from woost.extensions.blocks.block import Block


class ImageGalleryBlock(Block):

    instantiable = True
    view_class = "woost.views.ImageGallery"

    members_order = [
        "images",
        "gallery_type",
        "thumbnail_factory",
        "close_up_factory",
        "close_up_preload",
        "auto_play"
    ]

    images = schema.Collection(
        items = schema.Reference(type = File),
        relation_constraints = [File.resource_type.equal("image")],
        related_end = schema.Collection(),
        member_group = "content"
    )

    gallery_type = schema.String(
        required = True,
        default = "thumbnails",
        enumeration = ["thumbnails", "slideshow"],
        edit_control = "cocktail.html.RadioSelector",
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
               else translations(
                   "ImageGalleryBlock.gallery_type-%s" % value,
                   language,
                   **kwargs
               ),
        member_group = "content"
    )

    thumbnail_factory = ImageFactoryMember(
        required = True,
        default = "image_gallery_thumbnail",
        enumeration = lambda ctx:
            templates.get_class("woost.views.ImageGallery")
            .thumbnail_sizes.keys(),
        member_group = "content"
    )

    close_up_factory = ImageFactoryMember(
        required = True,
        default = "image_gallery_close_up",
        enumeration = lambda ctx:
            templates.get_class("woost.views.ImageGallery")
            .close_up_sizes.keys(),
        member_group = "content"
    )

    close_up_preload = schema.Boolean(
        required = True,
        default = True,
        member_group = "content"
    )

    auto_play = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    labels_visible = schema.Boolean(
        required = True,
        default = True,
        member_group = "content"
    )

    footnotes_visible = schema.Boolean(
        required = True,
        default = True,
        member_group = "content"
    )

    original_link_visible = schema.Boolean(
        required = True,
        default = False,
        member_group = "content"
    )

    close_up_enabled = schema.Boolean(
        required = True,
        default = True,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.images = self.images
        view.gallery_type = self.gallery_type
        view.thumbnail_factory = self.thumbnail_factory
        view.close_up_factory = self.close_up_factory
        view.slider_options["auto"] = self.auto_play
        view.labels_visible = self.labels_visible
        view.footnotes_visible = self.footnotes_visible
        view.original_link_visible = self.original_link_visible
        view.close_up_enabled = self.close_up_enabled
        view.close_up_preload = self.close_up_preload




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.translations import translations

translations.define("Woost installation",
    en = u"Woost installation"
)

translations.define("Installer.project",
    en = u"Project"
)

translations.define("Installer.project_name",
    en = u"Project name"
)

translations.define("Installer.project_name-explanation",
    en = u"""Should be in CamelCase form. Will be used as a prefix for Python
    class names, and transformed to lowercase to determine the name of the
    Python package assigned to the project.
    """
)

translations.define("Installer.python_package_repository",
    en = u"Package installation path"
)

translations.define("Installer.python_package_repository-explanation",
    en = u"""Location were the project's Python package and data files will be
    installed."""
)

translations.define("Installer.admin_email",
    en = u"Administrator email"
)

translations.define("Installer.admin_password",
    en = u"Administrator password"
)

translations.define("Installer.languages",
    en = u"Languages"
)

translations.define("Installer.languages-explanation",
    en = u"A list of ISO language codes, separated by whitespace."
)

translations.define("Installer.template_engine",
    en = u"Default template engine"
)

translations.define("Installer.webserver",
    en = u"Web server"
)

translations.define("Installer.webserver_host",
    en = u"Web server host"
)

translations.define("Installer.webserver_port",
    en = u"Web server port number"
)

translations.define("Installer.validate_webserver_address",
    en = u"Test address availability"
)

translations.define("Installer.database",
    en = u"Database"
)

translations.define("Installer.database_host",
    en = u"Database host"
)

translations.define("Installer.database_port",
    en = u"Database port number"
)

translations.define("Installer.validate_database_address",
    en = u"Test address availability"
)

translations.define("Install",
    en = u"Install"
)

translations.define("Installation successful",
    en = u"Your new site has been installed successfully."
)

translations.define(
    "woost.controllers.installer.InstallFolderExists-instance",
    ca = u"El directori d'instal·lació seleccionat ja existeix",
    es = u"El directorio de instalación seleccionado ya existe",
    en = u"The chosen installation directory already exists"
)

translations.define(
    "woost.controllers.installer no installation paths available",
    en = u"""The installer needs to create and import a Python package for the
    project, but currently there's no location available where the package
    could reside. Please, set up a PYTHONPATH environment variable and restart
    the installation process."""
)

translations.define(
    "woost.controllers.installer.WrongAddressError-instance",
    en = lambda instance:
        u"The indicated <em>%s</em> and <em>%s</em> combination is not "
        u"available on this server"
        % (
            translations(instance.host_member, "en"),
            translations(instance.port_member, "en")
        )
)

translations.define(
    "woost.controllers.installer.SubprocessError-instance",
    en = lambda instance:
        u"The installer found an unexpected error and couldn't continue. "
        u"The <em>%s</em> process returned the following error: <pre>%s</pre>"
        % (instance.cmd, instance.error_output)
)

translations.define(
    "Unknown error",
    en = lambda error: u"Unexpected <em>%s</em> exception: %s"
        % (error.__class__.__name__, error)
)




#-*- coding: utf-8 -*-
u"""Defines the `PasswordChangeConfirmationController` class.

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""
import cherrypy
from cocktail import schema
from cocktail.events import event_handler
from cocktail.persistence import datastore
from cocktail.controllers import (
    request_property,
    get_parameter,
    FormProcessor,
    Form
)
from woost.models.user import User
from woost.controllers.backoffice.usereditnode import PasswordConfirmationError
from woost.controllers.documentcontroller import DocumentController
from woost.controllers.passwordchangecontroller import generate_confirmation_hash


class PasswordChangeConfirmationController(FormProcessor, DocumentController):

    is_transactional = True
    class_view = "woost.views.PasswordChangeFormTemplate"

    @event_handler
    def handle_traversed(cls, e):

        controller = e.source

        # Make sure we are given a user
        if controller.user is None:
            raise cherrypy.HTTPError(400, "Invalid user")

        # Verify the request using the provided hash
        provided_hash = controller.hash
        expected_hash = generate_confirmation_hash(controller.identifier)

        if provided_hash != expected_hash:
            raise cherrypy.HTTPError(400, "Invalid hash")

    @request_property
    def identifier_member(self):
        cms = self.context["cms"]
        return cms.authentication.identifier_field

    @request_property
    def identifier(self):
        member = self.identifier_member.copy()
        member.name = "user"
        return get_parameter(member, errors = "ignore")

    @request_property
    def hash(self):
        return get_parameter(schema.String("hash"))

    @request_property
    def user(self):
        if self.identifier:
            return User.get_instance(**{
                self.identifier_member.name: self.identifier
            })

    class ChangePasswordForm(Form):

        @request_property
        def schema(self):

            form_schema = schema.Schema("PasswordChangeConfirmationForm")

            # New password
            password_member = User.password.copy()
            password_member.required = True
            form_schema.add_member(password_member)

            # New password confirmation
            password_confirmation_member = schema.String(
                name = "password_confirmation",
                edit_control = "cocktail.html.PasswordBox",
                required = True
            )

            @password_confirmation_member.add_validation
            def validate_password_confirmation(member, value, ctx):
                password = ctx.get_value("password")
                password_confirmation = value

                if password and password_confirmation \
                and password != password_confirmation:
                    yield PasswordConfirmationError(member, value, ctx)

            form_schema.add_member(password_confirmation_member)

            return form_schema

        def submit(self):

            Form.submit(self)

            # Update the user's password
            user = self.controller.user
            user.password = self.data["password"]
            datastore.commit()

            # Log in the user (after all, we just made certain it's him/her)
            self.controller.context["cms"].authentication.set_user_session(user)

    @request_property
    def output(self):
        output = DocumentController.output(self)
        output["identifier"] = self.identifier
        output["hash"] = self.hash
        return output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail import schema
from woost.models.document import Document

class News(Document):

    members_order = [
        "summary",
        "body"
    ]

    summary = schema.String(
        edit_control = "woost.views.RichTextEditor",
        listed_by_default = False,
        translated = True,
        member_group = "content"
    )

    body = schema.String(
        edit_control = "woost.views.RichTextEditor",
        listed_by_default = False,
        translated = True,
        member_group = "content"
    )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail import schema
from cocktail.modeling import getter
from cocktail.iteration import first
from cocktail.events import event_handler
from cocktail.controllers.renderingengines import get_rendering_engine
from woost.models.publishable import Publishable
from woost.models.file import File
from woost.models.controller import Controller


class Document(Publishable):

    instantiable = True
    default_per_language_publication = True

    groups_order = [
        "content", "navigation", "presentation", "publication", "meta",
        "robots"
    ]

    members_order = (
        "title",
        "inner_title",
        "template",
        "description",
        "keywords",
        "attachments",
        "page_resources",
        "branch_resources",
        "children",
        "robots_should_index",
        "robots_should_follow"
    )

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.document_controller")
    )

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        required = True,
        member_group = "content"
    )

    inner_title = schema.String(
        translated = True,
        listed_by_default = False,
        member_group = "content"
    )

    description = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    keywords = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    template = schema.Reference(
        type = "woost.models.Template",
        bidirectional = True,
        listed_by_default = False,
        member_group = "presentation"
    )

    branch_resources = schema.Collection(
        items = schema.Reference(
            type = Publishable,
            required = True,
            relation_constraints =
                Publishable.resource_type.equal("html_resource")
        ),
        related_end = schema.Collection()
    )

    page_resources = schema.Collection(
        items = schema.Reference(
            type = Publishable,
            required = True,
            relation_constraints =
                Publishable.resource_type.equal("html_resource")
        ),
        related_end = schema.Collection()
    )

    attachments = schema.Collection(
        items = schema.Reference(
            type = Publishable,
            required = True
        ),
        selector_default_type = File,
        related_end = schema.Collection()
    )

    children = schema.Collection(
        items = "woost.models.Publishable",
        bidirectional = True,
        related_key = "parent",
        cascade_delete = True
    )

    robots_should_index = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "robots"
    )

    robots_should_follow = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "robots"
    )

    def _update_path(self, parent, path):

        Publishable._update_path(self, parent, path)

        if self.children:
            for child in self.children:
                child._update_path(self, child.path)

    def descend_tree(self, include_self = False):

        if include_self:
            yield self

        if self.children:
            for child in self.children:
                for descendant in child.descend_tree(True):
                    yield descendant

    @getter
    def resources(self):
        """Iterates over all the resources that apply to the page.
        @type: L{Publishable}
        """
        for resource in self.inherited_resources:
            yield resource

        for resource in self.branch_resources:
            yield resource

        for resource in self.page_resources:
            yield resource

    def render(self, **values):
        """Renders the document using its template."""
        if self.template is None:
            raise ValueError("Can't render a document without a template")

        values["publishable"] = self
        engine = get_rendering_engine(self.template.engine)
        return engine.render(values, template = self.template.identifier)

    @property
    def default_image(self):
        return first(
            attachment
            for attachment in self.attachments
            if attachment.resource_type == "image"
            and attachment.is_accessible()
        )




#-*- coding: utf-8 -*-
u"""Defines the `URLShortener` model.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import abstractmethod
from cocktail import schema
from woost.models import Item


class URLShortener(Item):
    """Base model for all URL shortening services."""

    visible_from_root = False
    instantiable = False

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        descriptive = True
    )

    @abstractmethod
    def shorten_url(self, url):
        pass


class URLShortenerServiceError(Exception):
    """An exception raised if an error is reported during a request to an URL
    shortening service.
    """

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
import cherrypy
from cocktail.modeling import cached_getter
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.googlesearch import GoogleSearchExtension


class GoogleSearchController(DocumentController):

    @cached_getter
    def output(self):
        query = cherrypy.request.params.get("query") or u""
        page = cherrypy.request.params.get("page") or 0
        results = GoogleSearchExtension.instance.search(query, int(page))

        output = DocumentController.output(self)
        output["query"] = query
        output["results"] = results
        return output




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.translations import translations

translations.define("googlesearch next",
    es = u"Siguiente",
    en = u"Next",
    fr = u"Suivante",
    ca = u"Següent"
)

translations.define("googlesearch previous",
    es = u"Anterior",
    en = u"Previous",
    fr = u"Dernière",
    ca = u"Anterior"
)



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
import cherrypy
from cocktail.modeling import cached_getter
from cocktail.events import event_handler

from woost.models import Item
from woost.controllers.backoffice.contentviews import ContentViewsRegistry

from woost.controllers.backoffice.contentcontroller \
    import ContentController

from woost.controllers.backoffice.itemfieldscontroller \
    import ItemFieldsController


class FeedFieldsController(ItemFieldsController, ContentController):

    @cached_getter
    def persistence_prefix(self):
        return "Feed%s" % self.stack_node.item.id

    def _handle_form_data(self):

        form_data = self.stack_node.form_data
        query_parameters = form_data["query_parameters"]
        ItemFieldsController._handle_form_data(self)
        form_data["query_parameters"] = query_parameters

        source = self.query_parameters_source
        query_parameters.update(
            type = source.get("type"),
            order = source.get("order"),
            filter = source.get("filter")
        )
        for key, value in source.iteritems():
            if key.startswith("filter_"):
                query_parameters[key] = value

    @cached_getter
    def query_parameters_source(self):
        source = self.stack_node.form_data["query_parameters"].copy()
        source.update(cherrypy.request.params)
        return source

    @cached_getter
    def user_collection(self):
        user_collection = ContentController.user_collection(self)
        user_collection.__class__.type.clear(user_collection)
        user_collection.params.source = self.query_parameters_source.get
        user_collection.content_views_registry = ContentViewsRegistry()
        user_collection.content_views_registry.add(
            Item,
            "woost.views.FlatContentView",
            is_default = True
        )
        return user_collection

    @cached_getter
    def output(self):
        output = ContentController.output(self)
        output.update(ItemFieldsController.output(self))
        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.events import when
from cocktail import schema
from cocktail.translations import translations
from cocktail.controllers import context
from woost.models import Extension


translations.define("StaticSiteExtension",
    ca = u"Lloc web estàtic",
    es = u"Sitio web estático",
    en = u"Static website"
)

translations.define("StaticSiteExtension-plural",
    ca = u"Lloc web estàtic",
    es = u"Sitio web estático",
    en = u"Static website"
)

translations.define("StaticSiteExtension.file_extension",
    ca = u"Extensió de fitxer",
    es = u"Extensión de fichero",
    en = u"File extension"
)


class StaticSiteExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet exportar una versió estàtica del lloc web.""",
            "ca"
        )
        self.set("description",
            u"""Permite exportar una versión estática del sitio web.""",
            "es"
        )
        self.set("description",
            u"""Exports the site using static HTML pages.""",
            "en"
        )

    def _load(self):

        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController

        from woost.extensions.staticsite import (
            useraction,
            strings,
            staticsitesnapshoter,
            exportationpermission,
            exportstaticsitecontroller
        )

        from woost.extensions.staticsite.staticsitesnapshoter \
            import StaticSiteSnapShoter
        from woost.extensions.staticsite.staticsitedestination \
            import StaticSiteDestination

        BackOfficeController.export_static = \
            exportstaticsitecontroller.ExportStaticSiteController

        StaticSiteExtension.add_member(
            schema.Collection(
                "snapshoters",
                items = schema.Reference(
                    type = StaticSiteSnapShoter,
                ),
                bidirectional = True,
                integral = True,
                related_end = schema.Reference(),
                min = 1
            )
        )

        StaticSiteExtension.add_member(
            schema.Collection(
                "destinations",
                items = schema.Reference(
                    type = StaticSiteDestination,
                ),
                bidirectional = True,
                integral = True,
                related_end = schema.Reference(),
                min = 1
            )
        )

        # Disable interactive features from rendered pages when rendering
        # static content
        from woost.controllers.cmscontroller import CMSController

        @when(CMSController.producing_output)
        def disable_user_controls(event):
            if context.get("exporting_static_site", False):
                event.output["show_user_controls"] = False




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail import schema
from cocktail.dateutils import split_seconds
from cocktail.translations import translations
from cocktail.html import Element, Content
from woost.models import Publishable, Controller

def _link_display(parent, obj, member):
    url = parent.get_member_value(obj, member)
    return (
        Content("-") if not url
        else Element("a", href = url, children = [url])
    )


class VimeoVideo(Publishable):

    instantiable = False

    groups_order = [
        "content",
        "video_file_data",
        "stats",
        "vimeo_data"
    ]

    members_order = [
        "title",
        "description",
        "vimeo_tags",
        "duration",
        "width",
        "height",
        "approval_count",
        "views_count",
        "comments_count",
        "vimeo_video_id",
        "uri",
        "vimeo_user_name",
        "vimeo_user_url",
        "upload_date"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.uri_controller")
    )

    # Thumbnail URIs
    small_thumbnail_uri = None
    medium_thumbnail_uri = None
    large_thumbnail_uri = None
    vimeo_user_small_portrait_uri = None
    vimeo_user_medium_portrait_uri = None
    vimeo_user_large_portrait_uri = None

    title = schema.String(
        member_group = "content",
        descriptive = True,
        translated = True,
        required = True
    )

    description = schema.String(
        member_group = "content",
        translated = True,
        edit_control = "woost.views.RichTextEditor"
    )

    vimeo_tags = schema.String(
        member_group = "content",
        editable = False
    )

    duration = schema.Integer(
        member_group = "video_file_data",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
            else translations("time span", span = split_seconds(value))
    )

    width = schema.Integer(
        member_group = "video_file_data",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    height = schema.Integer(
        member_group = "video_file_data",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    approval_count = schema.Integer(
        member_group = "stats",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    views_count = schema.Integer(
        member_group = "stats",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    comments_count = schema.Integer(
        member_group = "stats",
        required = True,
        min = 0,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    vimeo_video_id = schema.String(
        member_group = "vimeo_data",
        unique = True,
        required = True,
        indexed = True,
        editable = False,
        text_search = False,
        listed_by_default = False
    )

    uri = schema.String(
        member_group = "vimeo_data",
        required = True,
        editable = False,
        listed_by_default = False,
        display = _link_display
    )

    vimeo_user_name = schema.String(
        member_group = "vimeo_data",
        required = True,
        indexed = True,
        editable = False,
        listed_by_default = False
    )

    vimeo_user_url = schema.String(
        member_group = "vimeo_data",
        required = True,
        editable = False,
        listed_by_default = False,
        display = _link_display
    )

    upload_date = schema.DateTime(
        member_group = "vimeo_data",
        required = True,
        editable = False,
        indexed = True,
        listed_by_default = False
    )

    def get_embed_uri(self):
        return "http://player.vimeo.com/video/%s" % self.vimeo_video_id




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""

class TwitterAPIError(Exception):
    """An exception raised if an error is reported by Twitter's REST API."""

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
import cherrypy
from campaign_monitor_api import CampaignMonitorApi
from cocktail import schema
from cocktail.modeling import cached_getter
from cocktail.pkgutils import import_object
from cocktail.controllers import FormProcessor, Form
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.campaignmonitor import CampaignMonitorExtension


class CampaignMonitorController(FormProcessor, DocumentController):

    class SubscriptionForm(Form):

        campaign_monitor_errors = None

        @cached_getter
        def model(self):
            return import_object(self.controller.context["publishable"].subscription_form)

        @cached_getter
        def schema(self):
            form_schema = Form.schema(self)
            publishable = self.controller.context["publishable"]

            form_schema.add_member(schema.Reference(
                "list",
                type = "woost.extensions.campaignmonitor.campaignmonitorlist."
                    "CampaignMonitorList",
                required = True,
                enumeration = self.available_lists,
                default = self.available_lists[0]
            ))

            return form_schema

        @cached_getter
        def available_lists(self):

            lists = getattr(
                self.controller.context["publishable"],
                "lists",
                None
            )

            if not lists:
                raise ValueError(
                    "No Campaign Monitor lists defined for %s" % self
                )

            return lists

        @cached_getter
        def custom_fields(self):
            return {}

        def submit(self):
            Form.submit(self)

            extension = CampaignMonitorExtension.instance
            cmlist = self.data["list"]
            email = self.data["email"].encode("utf-8")
            name = self.data.get("name")
            name = name.encode("utf-8") if name else ""

            api = CampaignMonitorApi(
                extension.api_key,
                extension.client_id
            )

            try:
                user_subscribed = api.subscribers_get_is_subscribed(
                    cmlist.list_id,
                    email
                )
            except CampaignMonitorApi.CampaignMonitorApiException:
                user_subscribed = False

            # Obtain custom fields
            new_custom_fields = self.custom_fields

            if user_subscribed:
                response = api.subscribers_get_single_subscriber(
                    cmlist.list_id,
                    email
                )
                subscriber = response.get("Subscribers.GetSingleSubscriber")
                custom_fields = subscriber[0].get("CustomFields")
            else:
                custom_fields = {}

            custom_fields.update(**new_custom_fields)

            # Encode custom fields
            encoded_custom_fields = {}
            for key, value in custom_fields.items():
                encoded_key = (
                    key.encode("utf-8") if isinstance(key, unicode) else key
                )
                encoded_value = (
                    value.encode("utf-8") if isinstance(value, unicode) else value
                )
                encoded_custom_fields[encoded_key] = encoded_value

            try:
                api.subscriber_add_and_resubscribe(
                    cmlist.list_id,
                    email,
                    name,
                    encoded_custom_fields
                )
            except CampaignMonitorApi.CampaignMonitorApiException:
                # TODO: Capture the error code and show the correct message
                self.campaign_monitor_errors = True
            else:

                if user_subscribed and cmlist.confirmation_success_page:
                    uri = self.get_confirmation_uri(cmlist, email = email, name = name, **encoded_custom_fields)
                elif not user_subscribed and cmlist.pending_page:
                    uri = self.get_pending_uri(cmlist, email = email, name = name, **encoded_custom_fields)
                else:
                    uri = self.get_default_uri(cmlist, email = email, name = name, **encoded_custom_fields)

                raise cherrypy.HTTPRedirect(uri.encode("utf-8"))

        def get_confirmation_uri(self, cmlist, **kwargs):
            return self.controller.context["cms"].uri(cmlist.confirmation_success_page)

        def get_pending_uri(self, cmlist, **kwargs):
            return self.controller.context["cms"].uri(cmlist.pending_page)

        def get_default_uri(self, cmlist, **kwargs):
            return self.controller.context["cms"].uri(self.controller.context["publishable"])





#-*- coding: utf-8 -*-
"""

@author:		Xavier Torras
@contact:		xavier.torras@whads.com
@organization:	Whads/Accent SL
@since:			Febrary 2010
"""
from datetime import datetime
from simplejson import loads
import urllib2
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail import schema
from woost.models import (
    Extension,
    get_current_user,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    PermissionExpression
)

translations.define("VimeoExtension",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)

translations.define("VimeoExtension-plural",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)

translations.define("VimeoExtension.default_vimeo_user_name",
    ca = u"Nom d'usuari Vimeo per defecte",
    es = u"Nombre de usuario Vimeo por defecto",
    en = u"Default Vimeo Username"
)


class VimeoExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet integrar vídeos allotjats a vimeo.com""",
            "ca"
        )
        self.set("description",
            u"""Permite integrar videos alojados en vimeo.com""",
            "es"
        )
        self.set("description",
            u"""Publish videos from vimeo.com""",
            "en"
        )

    default_vimeo_user_name = schema.String()

    def _load(self):

        # Load extension models, translations and UI extensions
        from woost.extensions.vimeo import (
            video,
            strings,
            useraction,
            vimeovideorenderer,
            migration
        )

        # Setup the synchronization view
        from woost.controllers.backoffice.backofficecontroller \
            import BackOfficeController
        from woost.extensions.vimeo.synccontroller import SyncVimeoController

        BackOfficeController.sync_vimeo = SyncVimeoController

    def synchronize(self, user_name, restricted = False):
        """Synchronizes the list of videos of the given Vimeo account with the
        site's database.

        The method queries Vimeo's public HTTP API, retrieving the list of
        videos for the indicated account and comparing it with the set of
        already known videos (from previous executions of this method). The
        local database will be updated as follows:

            * Videos declared by Vimeo that are not present in the database
              will generate new instances.
            * Videos that exist on both ends will be updated with the data
              provided by the Vimeo service (only non editable members will be
              updated, so that data entered by users in the backoffice is
              preserved).
            * Videos that were instantiated in a previous run but which have
              been deleted at the Vimeo side will be removed from the database.

        @param user_name: The Vimeo account to load the videos from.
        @type user_name: str

        @param restricted: Indicates if access control should be applied to the
            operations performed by the method.
        @type restricted: bool
        """
        from woost.extensions.vimeo.video import VimeoVideo

        if restricted:
            user = get_current_user()

        # Request the data from Vimeo's simple API
        service_uri = "http://vimeo.com/api/v2/%s/videos.json"
        request = urllib2.Request(service_uri % user_name)
        request.add_header('Referer', '')
        request.add_header('User-Agent', 'woost.extensions.vimeo')
        response = urllib2.urlopen(request).read()

        # Parse JSON data
        remote_videos = set()

        for video_data in loads(response):

            video_id = video_data["id"]
            video = VimeoVideo.get_instance(vimeo_video_id = video_id)
            remote_videos.add(video_id)

            # Check permissions
            if restricted and not user.has_permission(
                CreatePermission if video is None else ModifyPermission,
                target = (video or VimeoVideo)
            ):
                continue

            # Create new videos
            if video is None:
                is_new = True
                video = VimeoVideo()
                video.insert()
                video.vimeo_video_id = video_data["id"]
                video.vimeo_user_name = video_data["user_name"]
                video.title = video_data["title"]
                video.description = video_data["description"]

            # Modify new or updated videos
            video.small_thumbnail_uri = video_data["thumbnail_small"]
            video.medium_thumbnail_uri = video_data["thumbnail_medium"]
            video.large_thumbnail_uri = video_data["thumbnail_large"]
            video.vimeo_user_small_portrait_uri = video_data["user_portrait_small"]
            video.vimeo_user_medium_portrait_uri = video_data["user_portrait_medium"]
            video.vimeo_user_large_portrait_uri = video_data["user_portrait_large"]
            video.vimeo_user_url = video_data["user_url"]
            video.uri = video_data["url"]
            video.vimeo_tags = video_data["tags"]
            video.duration = int(video_data["duration"])
            video.width = int(video_data["width"])
            video.height = int(video_data["height"])
            video.upload_date = datetime.strptime(
                video_data["upload_date"],
                "%Y-%m-%d %H:%M:%S"
            )
            video.approval_count = int(video_data["stats_number_of_likes"])
            video.views_count = int(video_data["stats_number_of_plays"])
            video.comments_count = int(video_data["stats_number_of_comments"])

        # Delete videos that have been deleted from the user account
        missing_videos = VimeoVideo.select([
            VimeoVideo.vimeo_user_name.equal(user_name),
            VimeoVideo.vimeo_video_id.not_one_of(remote_videos),
        ])
        missing_videos.verbose = True

        if restricted:
            missing_videos.add_filter(
                PermissionExpression(user, DeletePermission)
            )

        missing_videos.delete_items()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from threading import local

_thread_data = local()

def get_current_user():
    """Gets the current user for the present context.

    The concept of an 'active' user is used in several places to bind certain
    operations (authorization, versioning...) to an individual.

    @rtype: L{User}
    """
    return getattr(_thread_data, "user", None)

def set_current_user(user):
    """Sets the current user for the present context. See L{get_current_user}
    for more information.

    @param user: The user to set as the active user.
    @type user: L{User}
    """
    _thread_data.user = user




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import cherrypy
from cocktail.controllers import request_property
from woost.controllers.publishablecontroller import PublishableController
from woost.extensions.shop.basket import Basket


class ProductController(PublishableController):
    """A controller that serves rendered pages."""

    @request_property
    def view_class(self):
        view_class = self.context["publishable"].view_class

        if view_class is None:
            raise cherrypy.NotFound()

        return view_class

    @request_property
    def output(self):
        output = PublishableController.output(self)
        output["shop_order"] = Basket.get()
        return output




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.translations import translations
from woost.models import Extension

translations.define("SitemapExtension",
    ca = u"Refinament de l'indexat web",
    es = u"Refinamiento del indexado web",
    en = u"Web crawler hints"
)

translations.define("SitemapExtension-plural",
    ca = u"Refinament de l'indexat web",
    es = u"Refinamiento del indexado web",
    en = u"Web crawler hints"
)


class SitemapExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Genera un mapa de documents per optimitzar l'indexat del lloc
            web, seguint l'estàndard sitemap.org.""",
            "ca"
        )
        self.set("description",
            u"""Genera un mapa de documentos para optimizar el indexado del
            sitio, siguiendo el estándard sitemap.org.""",
            "es"
        )
        self.set("description",
            u"""Generates a document map to optimize the indexing of the site
            by web crawlers, following the sitemap.org standard.""",
            "en"
        )

    def _load(self):
        from woost.extensions.sitemap import publishable, strings
        self.install()

    def _install():

        from woost.models import (
            Site,
            Publishable,
            Document,
            Controller,
            extension_translations
        )

        # Sitemap controller
        sitemap_controller = self._create_asset(
            Controller,
            "sitemap_controller",
            title = extension_translations,
            python_name =
                "woost.extensions.sitemap.sitemapcontroller.SitemapController"
        )

        # Sitemap document
        sitemap_doc = self._create_asset(
            Document,
            "sitemap_document",
            title = extension_translations,
            parent = Site.main.home,
            path = "sitemap_xml",
            per_language_publication = False,
            mime_type = "text/xml",
            hidden = True,
            sitemap_indexable = False,
            controller = sitemap_controller
        )

        # Force indexing of the 'sitemap_indexable' member
        # (can't rely on defaults when executing queries)
        for item in Publishable.select():
            if not hasattr(item, "_sitemap_indexable"):
                item.sitemap_indexable = \
                    item.sitemap_indexable and not item.hidden




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			April 2010
"""
import cherrypy
from datetime import datetime
from campaign_monitor_api import CampaignMonitorApi
from cocktail import schema
from cocktail.modeling import cached_getter
from cocktail.controllers import context
from woost.models import Site, StandardPage
from woost.controllers.documentcontroller import DocumentController
from woost.extensions.campaignmonitor import CampaignMonitorExtension
from woost.extensions.campaignmonitor.campaignmonitorsubscriptionpage \
    import CampaignMonitorSubscriptionPage


class CampaignMonitorUnsubscriptionController(DocumentController):

    subscriber = None
    action = None
    max_seconds = 60 * 60

    def __init__(self, *args, **kwargs):
        self.subscriber = self.params.read(schema.String("user"))
        self.action = self.params.read(schema.String("action"))

        DocumentController.__init__(self, *args, **kwargs)

    def submit(self, *args, **kwargs):

        if self.action == "resubscribe" and self.subscriber:
            self.resubscribe(self.subscriber)


    def resubscribe(self, email):
        extension = CampaignMonitorExtension.instance

        api = CampaignMonitorApi(
            extension.api_key,
            extension.client_id
        )

        lists = api.client_get_lists()

        resubscribed_lists = 0
        cm_context = {
            "email": email,
            "lists": []
        }
        for i, list in enumerate(lists):
            try:
                response = api.subscribers_get_single_subscriber(
                    list.get("ListID"),
                    email.encode("utf-8")
                )
            except CampaignMonitorApi.CampaignMonitorApiException:
                continue
            else:
                subscriber = response.get("Subscribers.GetSingleSubscriber")
                name = subscriber[0].get("Name")
                date = subscriber[0].get("Date")
                state = subscriber[0].get("State")

                cm_context["lists"].append({
                    "list_id": list.get("ListID"),
                    "name": name,
                    "state": state,
                    "date": date
                })

                if state == "Unsubscribed":
                    date = datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
                    now = datetime.now()

                    diff = now - date

                    if date > now or diff.seconds < self.max_seconds:
                        custom_fields = subscriber[0].get("CustomFields")

                        # Encode custom fields
                        encoded_custom_fields = {}
                        for key, value in custom_fields.items():
                            encoded_key = (
                                key.encode("utf-8") if isinstance(key, unicode) else key
                            )
                            encoded_value = (
                                value.encode("utf-8") if isinstance(value, unicode) else value
                            )
                            encoded_custom_fields[encoded_key] = encoded_value

                            cm_context["lists"][i].update(**encoded_custom_fields)

                        try:
                            # Resubscribe
                            resubscribed_lists += 1
                            api.subscriber_add_and_resubscribe(
                                list.get("ListID"),
                                email.encode("utf-8"),
                                name.encode("utf-8") if name else name,
                                encoded_custom_fields
                            )
                        except CampaignMonitorApi.CampaignMonitorApiException:
                            self.campaign_monitor_errors = True

        uri = None

        if resubscribed_lists == 0:
            uri = self.get_subscription_uri(**cm_context)
        else:
            uri = self.get_pending_uri(**cm_context)

        if uri is None:
            uri = context["cms"].uri(
                Site.main.home
            )

        raise cherrypy.HTTPRedirect(uri.encode("utf-8"))

    def get_subscription_uri(self, **kwargs):
        subscription_page = CampaignMonitorSubscriptionPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_page"
        )
        if subscription_page:
            return context["cms"].uri(subscription_page)

    def get_pending_uri(self, **kwargs):
        pending_page = StandardPage.get_instance(
            qname = u"woost.extensions.campaignmonitor.pending_page"
        )
        if pending_page:
            return context["cms"].uri(pending_page)


    @cached_getter
    def output(self):
        output = DocumentController.output(self)
        output.update(
            subscriber = self.subscriber
        )

        return output



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
import cherrypy
from simplejson import dumps
from cocktail.events import event_handler
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers import request_property, get_parameter
from woost.models import Item
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController, EditNode


class OrderController(BaseBackOfficeController):

    @request_property
    def handling_ajax_request(self):
        return self.rendering_format == "json"

    @request_property
    def content_type(self):
        return self.member.items.type

    @request_property
    def collection(self):
        return schema.get(self.edit_node.form_data, self.member)

    @request_property
    def member(self):
        key = get_parameter(schema.String("member"))
        return self.item.__class__.get_member(key) if key else None

    @request_property
    def item(self):
        return self.edit_node.item

    @request_property
    def selection(self):
        return get_parameter(
            schema.Collection("selection",
                items = schema.Reference(type = self.content_type)
            )
        )

    @request_property
    def position(self):
        return get_parameter(
            schema.Integer("position",
                min = 0,
                max = len(self.collection)
            )
        )

    @request_property
    def action(self):
        return get_parameter(schema.String("action"))

    @request_property
    def ready(self):
        return self.selection and self.position is not None

    def submit(self):

        collection = self.collection
        selection = self.selection
        position = self.position
        size = len(collection)
        selection_size = len(selection)

        if position < 0:
            position = size + position

        for item in selection:
            collection.remove(item)

        for item in selection:
            collection.insert(position, item)
            position += 1

    def handle_error(self, error):
        if self.handling_ajax_request:
            self.output["error"] = translations(error)
        else:
            BaseBackOfficeController.handle_error(self, error)

    @event_handler
    def handle_after_request(cls, event):

        controller = event.source

        if not controller.handling_ajax_request:
            if controller.action == "cancel" or controller.successful:
                if controller.edit_stack:
                    controller.edit_stack.go(-2)
                else:
                    raise cherrypy.HTTPRedirect(controller.relative_uri())

    view_class = "woost.views.BackOfficeOrderView"

    @request_property
    def output(self):
        if self.handling_ajax_request:
            return {}
        else:
            output = BaseBackOfficeController.output(self)
            output.update(
                content_type = self.content_type,
                collection = self.collection,
                selection = self.selection
            )
            return output




#-*- coding: utf-8 -*-
u"""Provides the `UploadForm` class, that makes it easy to upload files into
`woost.models.File` objects.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from tempfile import mkdtemp
from shutil import move, rmtree
from cocktail import schema
from cocktail.controllers import Form, FileUpload, request_property
from woost.models import File
from woost.controllers import async_uploader


class UploadForm(Form):

    upload_options = {
        "async": True,
        "async_uploader": async_uploader,
        "async_upload_url": "/async_upload"
    }

    @request_property
    def upload_members(self):
        is_file_ref = (
            lambda member:
                isinstance(member, schema.Reference)
                and issubclass(member.type, File)
        )
        return [member
                for member in self.model.members().itervalues()
                if is_file_ref(member)
                or (
                    isinstance(member, schema.Collection)
                    and is_file_ref(member.items)
                )]

    @request_property
    def adapter(self):
        adapter = Form.adapter(self)

        for member in self.upload_members:
            key = member.name
            export_rule = self.ExportUploadInfo(self, key)
            export_rule.upload_options = self.upload_options
            adapter.export_rules.add_rule(export_rule)
            adapter.import_rules.add_rule(self.ImportUploadInfo(self, key))

        return adapter

    class ExportUploadInfo(schema.Rule):

        upload_options = {}

        def __init__(self, form, key):
            self.form = form
            self.key = key

        def adapt_schema(self, context):

            if context.consume(self.key):
                source_member = context.source_schema[self.key]
                target_member = FileUpload(
                    required = source_member.required,
                    hash_algorithm = "md5",
                    get_file_destination = self.form.get_temp_path
                )

                for key, value in self.upload_options.iteritems():
                    setattr(target_member, key, value)

                if isinstance(source_member, schema.Collection):
                    target_member = schema.Collection(
                        items = target_member,
                        min = source_member.min,
                        max = source_member.max
                    )

                target_member.name = self.key
                target_member.member_group = source_member.member_group
                target_member.adaptation_source = source_member
                target_member.copy_source = source_member
                context.target_schema.add_member(target_member)

        def adapt_object(self, context):
            if context.consume(self.key):
                value = context.get(self.key)
                if value:
                    source_member = context.source_schema[self.key]
                    if isinstance(source_member, schema.Collection):
                        adapted_value = [
                            self.export_upload(context, item)
                            for item in value
                        ]
                    else:
                        adapted_value = self.export_upload(context, value)
                    context.set(self.key, adapted_value)

        def export_upload(self, context, file):
            return {
                "file_name": file.file_name,
                "mime_type": file.mime_type,
                "file_size": file.file_size,
                "file_hash": file.file_hash
            }

    class ImportUploadInfo(schema.Rule):

        def __init__(self, form, key):
            self.form = form
            self.key = key

        def adapt_object(self, context):

            if context.consume(self.key):
                value = context.get(self.key, None)

                if value is not None:
                    source_member = context.source_schema[self.key]

                    if isinstance(source_member, schema.Collection):
                        adapted_value = [
                            self.import_upload(context, upload)
                            for upload in value
                        ]
                    else:
                        adapted_value = self.import_upload(context, value)

                    context.set(self.key, adapted_value)

        def import_upload(self, context, upload):

            member = context.target_schema[self.key]
            file = None

            if not isinstance(member, schema.Collection):
                file = context.target_object.get(self.key)

            if file is None:
                file = self.create_file(context)

            file.file_name = upload["file_name"]
            file.mime_type = upload["mime_type"]
            file.file_size = upload["file_size"]
            file.file_hash = upload["file_hash"]

            self.form.temp_paths[file] = self.form.get_temp_path(upload)

            return file

        def create_file(self, context):
            member = context.target_schema[self.key]
            return member.related_type()

    def submit(self):
        Form.submit(self)
        self.upload()

    @request_property
    def temp_upload_folder(self):
        return mkdtemp()

    @request_property
    def temp_paths(self):
        return {}

    def get_temp_path(self, upload):
        temp_path = upload.get("temp_path")
        if temp_path is None:
            upload["temp_path"] = temp_path = os.path.join(
                self.temp_upload_folder,
                str(id(upload))
            )
        return temp_path

    def upload(self):

        # Move uploaded files to their permanent location
        try:
            for member in self.upload_members:

                value = schema.get(self.instance, member)

                if value is None:
                    continue

                if isinstance(member, schema.Collection):
                    files = value
                else:
                    files = (value,)

                for i, file in enumerate(files):
                    temp_file = self.temp_paths[file]

                    if os.path.exists(temp_file):

                        dest = file.file_path

                        if os.path.exists(dest):
                            os.remove(dest)

                        move(temp_file, dest)

        # Remove the temporary folder
        finally:
            rmtree(self.temp_upload_folder)




#-*- coding: utf-8 -*-
u"""
(X)HTML templates for the CMS backend application.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from os import path as _path




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail.translations import get_language, translations
from cocktail import schema
from cocktail.events import Event, event_handler
from woost.models import (
    Item,
    Site,
    User,
    get_current_user,
    ModifyMemberPermission
)
from woost.extensions.locations.location import Location
from woost.extensions.ecommerce.ecommercebillingconcept \
    import ECommerceBillingConcept
from woost.extensions.payments import PaymentsExtension

def _translate_amount(amount, language = None, **kwargs):
    if amount is None:
        return ""
    else:
        return translations(
            amount.quantize(Decimal("1.00")),
            language,
            **kwargs
        )

def _get_default_payment_type():
    from woost.extensions.ecommerce import ECommerceExtension
    payment_types = ECommerceExtension.instance.payment_types
    if len(payment_types) == 1:
        return payment_types[0]


class ECommerceOrder(Item):

    payment_types_completed_status = {
        "payment_gateway": "accepted",
        "transfer": "payment_pending",
        "cash_on_delivery": "payment_pending"
    }

    incoming = Event(doc = """
        An event triggered when a new order is received.
        """)

    completed = Event(doc = """
        An event triggered when an order is completed.
        """)

    groups_order = [
        "shipping_info",
        "billing"
    ]

    members_order = [
        "customer",
        "address",
        "town",
        "region",
        "country",
        "postal_code",
        "language",
        "status",
        "purchases",
        "payment_type",
        "total_price",
        "pricing",
        "total_shipping_costs",
        "shipping_costs",
        "total_taxes",
        "taxes",
        "total"
    ]

    customer = schema.Reference(
        type = User,
        related_end = schema.Collection(),
        required = True,
        default = schema.DynamicDefault(get_current_user)
    )

    address = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    town = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    region = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    country = schema.Reference(
        member_group = "shipping_info",
        type = Location,
        relation_constraints = [Location.location_type.equal("country")],
        default_order = "location_name",
        related_end = schema.Collection(),
        required = True,
        listed_by_default = False,
        user_filter = "cocktail.controllers.userfilter.MultipleChoiceFilter"
    )

    postal_code = schema.String(
        member_group = "shipping_info",
        required = True,
        listed_by_default = False
    )

    language = schema.String(
        required = True,
        format = "^[a-z]{2}$",
        editable = False,
        default = schema.DynamicDefault(get_language),
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs)
    )

    status = schema.String(
        required = True,
        indexed = True,
        enumeration = [
            "shopping",
            "payment_pending",
            "accepted",
            "failed",
            "refund"
        ],
        default = "shopping",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "ECommerceOrder.status-" + value,
                language,
                **kwargs
            )
    )

    purchases = schema.Collection(
        items = "woost.extensions.ecommerce.ecommercepurchase."
                "ECommercePurchase",
        integral = True,
        bidirectional = True,
        min = 1
    )

    payment_type = schema.String(
        member_group = "billing",
        required = True,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "ECommerceOrder.payment_type-%s" % value,
                language = language
            ),
        default = schema.DynamicDefault(_get_default_payment_type),
        text_search = False,
        edit_control = "cocktail.html.RadioSelector",
        listed_by_default = False
    )

    total_price = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False,
        translate_value = _translate_amount
    )

    pricing = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total_shipping_costs = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False,
        translate_value = _translate_amount
    )

    shipping_costs = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total_taxes = schema.Decimal(
        member_group = "billing",
        editable = False,
        listed_by_default = False,
        translate_value = _translate_amount
    )

    taxes = schema.Collection(
        member_group = "billing",
        items = schema.Reference(type = ECommerceBillingConcept),
        related_end = schema.Collection(
            block_delete = True
        ),
        editable = False
    )

    total = schema.Decimal(
        member_group = "billing",
        editable = False,
        translate_value = _translate_amount
    )

    def calculate_cost(self,
        apply_pricing = True,
        apply_shipping_costs = True,
        apply_taxes = True
    ):
        """Calculates the costs for the order.
        :rtype: dict
        """
        from woost.extensions.ecommerce import ECommerceExtension
        extension = ECommerceExtension.instance

        order_costs = {
            "price": {
                "cost": Decimal("0.00"),
                "percentage": Decimal("0.00"),
                "concepts": []
            },
            "shipping_costs": {
                "cost": Decimal("0.00"),
                "percentage": Decimal("0.00"),
                "concepts": []
            },
            "taxes": {
                "cost": Decimal("0.00"),
                "percentage": Decimal("0.00"),
                "concepts": []
            },
            "purchases": {}
        }

        # Per purchase costs:
        for purchase in self.purchases:
            purchase_costs = purchase.calculate_costs(
                apply_pricing = apply_pricing,
                apply_shipping_costs = apply_shipping_costs,
                apply_taxes = apply_taxes
            )
            order_costs["purchases"][purchase] = purchase_costs

            order_costs["price"]["cost"] += purchase_costs["price"]["total"]
            order_costs["shipping_costs"]["cost"] += \
                purchase_costs["shipping_costs"]["total"]
            order_costs["taxes"]["cost"] += purchase_costs["taxes"]["total"]

        # Order price
        order_price = order_costs["price"]

        if apply_pricing:
            for pricing in extension.pricing:
                if pricing.applies_to(self):
                    pricing.apply(self, order_price)

        order_price["cost"] += \
            order_price["cost"] * order_price["percentage"] / 100

        order_price["total"] = order_price["cost"]

        # Order shipping costs
        order_shipping_costs = order_costs["shipping_costs"]

        if apply_shipping_costs:
            for shipping_cost in extension.shipping_costs:
                if shipping_cost.applies_to(self):
                    shipping_cost.apply(self, order_shipping_costs)

        order_shipping_costs["total"] = (
            order_shipping_costs["cost"]
            + order_price["total"] * order_shipping_costs["percentage"] / 100
        )

        # Order taxes
        order_taxes = order_costs["taxes"]

        if apply_taxes:
            for tax in extension.taxes:
                if tax.applies_to(self):
                    tax.apply(self, order_taxes)

        order_taxes["total"] = (
            order_taxes["cost"]
            + order_price["total"] * order_taxes["percentage"] / 100
        )

        # Total
        order_costs["total"] = (
            order_price["total"]
          + order_shipping_costs["total"]
          + order_taxes["total"]
        )

        return order_costs

    def update_cost(self,
        apply_pricing = True,
        apply_shipping_costs = True,
        apply_taxes = True
    ):
        costs = self.calculate_cost(
            apply_pricing = apply_pricing,
            apply_shipping_costs = apply_shipping_costs,
            apply_taxes = apply_taxes
        )

        self.total_price = costs["price"]["total"]
        self.pricing = list(costs["price"]["concepts"])

        self.total_shipping_costs = costs["shipping_costs"]["total"]
        self.shipping_costs = list(costs["shipping_costs"]["concepts"])

        self.total_taxes = costs["taxes"]["total"]
        self.taxes = list(costs["taxes"]["concepts"])

        self.total = costs["total"]

        for purchase, purchase_costs in costs["purchases"].iteritems():
            purchase.total_price = purchase_costs["price"]["total"]
            purchase.pricing = list(purchase_costs["price"]["concepts"])
            self.pricing.extend(purchase.pricing)

            purchase.total_shipping_costs = \
                purchase_costs["shipping_costs"]["total"]
            purchase.shipping_costs = \
                list(purchase_costs["shipping_costs"]["concepts"])
            self.shipping_costs.extend(purchase.shipping_costs)

            purchase.total_taxes = purchase_costs["taxes"]["total"]
            purchase.taxes = list(purchase_costs["taxes"]["concepts"])
            self.taxes.extend(purchase.taxes)

            purchase.total = purchase_costs["total"]

    def count_units(self):
        return sum(purchase.quantity for purchase in self.purchases)

    def get_weight(self):
        return sum(purchase.get_weight() for purchase in self.purchases)

    def add_purchase(self, purchase):
        for order_purchase in self.purchases:
            if order_purchase.__class__ is purchase.__class__ \
            and order_purchase.product is purchase.product \
            and all(
                order_purchase.get(option) == purchase.get(option)
                for option in purchase.get_options()
                if option.name != "quantity"
            ):
                order_purchase.quantity += purchase.quantity
                purchase.product = None
                if purchase.is_inserted:
                    purchase.delete()
                break
        else:
            self.purchases.append(purchase)

    @classmethod
    def get_public_schema(cls):
        public_schema = schema.Schema("OrderCheckoutSummary")
        cls.get_public_adapter().export_schema(
            cls,
            public_schema
        )

        payment_type = public_schema.get_member("payment_type")
        if payment_type:
            payments = PaymentsExtension.instance

            if payments.enabled and payments.payment_gateway:

                translate_value = payment_type.translate_value

                def payment_type_translate_value(value, language = None, **kwargs):
                    if value == "payment_gateway":
                        return payments.payment_gateway.label
                    else:
                        return translate_value(
                            value, language = language, **kwargs
                        )

                payment_type.translate_value = payment_type_translate_value

        return public_schema

    @classmethod
    def get_public_adapter(cls):
        from woost.extensions.ecommerce import ECommerceExtension

        user = get_current_user()
        adapter = schema.Adapter()
        adapter.exclude(["customer", "status", "purchases"])
        adapter.exclude([
            member.name
            for member in cls.members().itervalues()
            if not member.visible
            or not member.editable
            or not issubclass(member.schema, ECommerceOrder)
            or not user.has_permission(
                ModifyMemberPermission,
                member = member
            )
        ])
        if len(ECommerceExtension.instance.payment_types) == 1:
            adapter.exclude(["payment_type"])
        return adapter

    @property
    def is_completed(self):
        return self.status \
        and self.status == self.payment_types_completed_status.get(
            self.payment_type
        )

    @event_handler
    def handle_changed(cls, event):

        item = event.source
        member = event.member

        if member.name == "status":

            if event.previous_value == "shopping" \
            and event.value in ("payment_pending", "accepted"):
                item.incoming()

            if item.is_completed:
                item.completed()

    def get_description_for_gateway(self):
        if Site.main.site_name:
            return translations(
                "woost.extensions.ECommerceOrder description for gateway"
            ) % Site.main.site_name
        else:
            return translations(self)
